# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'CommunityType'
        db.create_table('peer_communitytype', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Description",max_length=32,unique=True))
        ))
        
        # Mock Models
        CommunityType = db.mock_model(model_name='CommunityType', db_table='peer_communitytype', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'Community'
        db.create_table('peer_community', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('community', models.CharField("Community",max_length=20,unique=True)),
            ('type', models.ForeignKey(CommunityType,verbose_name="Type")),
            ('description', models.CharField("Description",max_length=64))
        ))
        
        db.send_create_signal('peer', ['CommunityType','Community'])
    
    def backwards(self):
        db.delete_table('peer_community')
        db.delete_table('peer_communitytype')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Service Activator Daemon
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import os,logging,pty,signal,time,re,sys,signal,Queue,sets,cPickle,tempfile
from errno import ECONNREFUSED
from noc.sa.profiles import profile_registry
from noc.sa.script import script_registry,ScriptSocket
from noc.sa.rpc import RPCSocket,file_hash,get_digest
from noc.sa.protocols.sae_pb2 import *
from noc.lib.fileutils import safe_rewrite
from noc.lib.daemon import Daemon
from noc.lib.fsm import FSM,check_state
from noc.lib.nbsocket import ConnectedTCPSocket,SocketFactory,PTYSocket
from noc.lib.debug import DEBUG_CTX_CRASH_PREFIX
from threading import Lock

##
##
##
class Service(SAEService):
    def ping(self,controller,request,done):
        done(controller,response=PingResponse())
        
    def script(self,controller,request,done):
        def script_callback(script):
            if script.result:
                c=ScriptResponse()
                c.result=script.result
                done(controller,response=c)
            else:
                e=Error()
                e.code=ERR_SCRIPT_EXCEPTION
                e.text=script.error_traceback
                done(controller,error=e)
        try:
            profile=profile_registry[request.access_profile.profile]
        except:
            e=Error()
            e.code=ERR_INVALID_PROFILE
            e.text="Invalid profile '%s'"%request.access_profile.profile
            done(controller,error=e)
            return
        try:
            script_class=script_registry[request.script]
        except:
            e=Error()
            e.code=ERR_INVALID_SCRIPT
            e.text="Invalid script '%s'"%request.script
            done(controller,error=e)
            return
        if request.access_profile.scheme not in profile.supported_schemes:
            e=Error()
            e.code=ERR_INVALID_SCHEME
            e.text="Access scheme '%s' is not supported for profile '%s'"%(self.code_to_scheme(request.access_profile.scheme),
                request.access_profile.profile)
            done(controller,error=e)
            return
        if self.activator.factory.count_subclass_sockets(ScriptSocket)>=self.activator.config.getint("activator","max_pull_config"):
            e=Error()
            e.code=ERR_OVERLOAD
            e.text="script concurrent session limit reached"
            done(controller,error=e)
            return
        kwargs={}
        for a in request.kwargs:
            kwargs[str(a.key)]=a.value
        self.activator.run_script(request.script,request.access_profile,script_callback,**kwargs)
    
    def ping_check(self,controller,request,done):
        def ping_check_callback(unreachable):
            u=sets.Set(unreachable)
            r=PingCheckResponse()
            for a in request.addresses:
                if a in u:
                    r.unreachable.append(a)
                else:
                    r.reachable.append(a)
            done(controller,response=r)
        self.activator.ping_check([a for a in request.addresses],ping_check_callback)
##
##
##
class ActivatorSocket(RPCSocket,ConnectedTCPSocket):
    def __init__(self,factory,address,port,local_address=None):
        ConnectedTCPSocket.__init__(self,factory,address,port,local_address)
        RPCSocket.__init__(self,factory.activator.service)
        
    def activator_event(self,event):
        self.factory.activator.event(event)
    
    def on_connect(self):
        self.activator_event("connect")
    
    def on_close(self):
        self.activator_event("close")
    
    def on_conn_refused(self):
        self.activator_event("refused")
##
## External fping process.
## Runs fping, supplies a list of checked hosts
## and reads a list uf unreachable hosts
##
class FPingProbeSocket(PTYSocket):
    def __init__(self,factory,fping_path,addresses,callback):
        self.result=""
        # Write hosts list to temporary file
        h,self.tmp_path=tempfile.mkstemp()
        f=os.fdopen(h,"w")
        f.write("\n".join(addresses)+"\n")
        f.close()
        self.callback=callback
        # Fping requires root to read hosts from file. Run it through the wrapper
        PTYSocket.__init__(self,factory,["./scripts/stdin-wrapper",self.tmp_path,fping_path,"-A","-u"])
        
    def on_close(self):
        os.unlink(self.tmp_path)
        # fping issues duplicated addresses sometimes.
        # Remove duplicates
        r={}
        for u in [x.strip() for x in self.result.split("\n") if x.strip()]:
            r[u]=None
        self.callback(r.keys())
        
    def on_read(self,data):
        self.result+=data
##
## Activator supervisor and daemon
##
class Activator(Daemon,FSM):
    daemon_name="noc-activator"
    FSM_NAME="Activator"
    DEFAULT_STATE="IDLE"
    STATES={
        "IDLE": {
                "timeout" : "CONNECT",
                "close"   : "IDLE",
                },
        "CONNECT" : {
                "timeout" : "IDLE",
                "refused" : "IDLE",
                "close"   : "IDLE",
                "connect" : "CONNECTED",
                },
        "CONNECTED" : {
                "timeout" : "IDLE",
                "close"   : "IDLE",
                "register": "REGISTRED",
                "error"   : "IDLE",
        },
        "REGISTRED" : {
                "timeout" : "IDLE",
                "auth"    : "AUTHENTICATED",
                "close"   : "IDLE",
                "error"   : "IDLE",
        },
        "AUTHENTICATED" : {
                "establish" : "ESTABLISHED",
                "upgrade"   : "UPGRADE",
                "close"   : "IDLE",
        },
        "UPGRADE" : {
                "establish" : "ESTABLISHED",
                "close"     : "IDLE",
        },
        "ESTABLISHED" : {
                "close"   : "IDLE",
        }
    }
    def __init__(self):
        Daemon.__init__(self)
        self.activator_name=self.config.get("activator","name")
        logging.info("Running activator '%s'"%self.activator_name)
        self.stand_alone_mode=self.config.get("activator","software_update") and not os.path.exists(os.path.join("sa","sae.py"))
        self.service=Service()
        self.service.activator=self
        self.factory=SocketFactory(tick_callback=self.tick)
        self.factory.activator=self
        self.children={}
        self.sae_stream=None
        self.event_sources=sets.Set()
        self.trap_collector=None
        self.syslog_collector=None
        logging.info("Loading profile classes")
        profile_registry.register_all() # Should be performed from ESTABLISHED state
        script_registry.register_all()
        self.nonce=None
        FSM.__init__(self)
        self.next_filter_update=None
        self.next_crashinfo_check=None
        self.script_threads={}
        self.script_lock=Lock()
        self.script_call_queue=Queue.Queue()
        
    ##
    ## IDLE state 
    ##
    def on_IDLE_enter(self):
        if self.sae_stream:
            self.sae_stream.close()
            self.sae_stream=None
        if self.trap_collector:
            self.stop_trap_collector()
        if self.syslog_collector:
            self.stop_syslog_collector()
        self.set_timeout(5)
    ##
    ## CONNECT state
    ##
    def on_CONNECT_enter(self):
        self.set_timeout(10)
        self.sae_stream=ActivatorSocket(self.factory,self.config.get("sae","host"),self.config.getint("sae","port"),
            self.config.get("sae","local_address"))
    ##
    ## CONNECTED state
    ##
    def on_CONNECTED_enter(self):
        self.set_timeout(10)
        self.register()
    ##
    ## REGISTRED
    ##
    def on_REGISTRED_enter(self):
        self.set_timeout(10)
        self.auth()
    ##
    ## AUTHENTICATED
    ##
    def on_AUTHENTICATED_enter(self):
        if self.stand_alone_mode:
            self.event("upgrade")
        else:
            logging.info("In-bundle package. Skiping software updates")
            self.event("establish")
    ##
    ## UPGRADE
    ##
    def on_UPGRADE_enter(self):
        logging.info("Requesting software update")
        self.manifest()
    ##
    ## ESTABLISHED
    ##
    def on_ESTABLISHED_enter(self):
        to_refresh_filters=False
        self.next_filter_update=None
        if self.config.get("activator","listen_traps"):
            self.start_trap_collector()
            to_refresh_filters=True
        if self.config.get("activator","listen_syslog"):
            self.start_syslog_collector()
            to_refresh_filters=True
        if to_refresh_filters:
            self.get_event_filter()
        if self.stand_alone_mode:
            self.check_crashinfo()
    ##
    ##
    ##
    def start_trap_collector(self):
        logging.debug("Starting trap collector")
        from noc.sa.trapcollector import TrapCollector
        self.trap_collector=TrapCollector(self,self.resolve_address(self.config.get("activator","listen_traps")),162)
        
    def stop_trap_collector(self):
        if self.trap_collector:
            logging.debug("Stopping trap collector")
            self.trap_collector.close()
            self.trap_collector=None
    ##
    ##
    ##
    def start_syslog_collector(self):
        logging.debug("Starting syslog collector")
        from noc.sa.syslogcollector import SyslogCollector
        self.syslog_collector=SyslogCollector(self,self.resolve_address(self.config.get("activator","listen_syslog")),514)
        
    def stop_syslog_collector(self):
        if self.syslog_collector:
            logging.debug("Stopping syslog collector")
            self.syslog_collector.close()
            self.syslog_collector=None
    ##
    ## Script support
    ##
    def run_script(self,name,access_profile,callback,**kwargs):
        logging.info("Script %s(%s)"%(name,access_profile.address))
        pv,pos,sn=name.split(".",2)
        profile=profile_registry["%s.%s"%(pv,pos)]()        
        script=script_registry[name](profile,self,access_profile,**kwargs)
        self.script_lock.acquire()
        self.script_threads[script]=callback
        self.script_lock.release()
        script.start()
        
    def on_script_exit(self,script):
        self.script_lock.acquire()
        cb=self.script_threads[script]
        del self.script_threads[script]
        self.script_lock.release()
        cb(script)
        
    def request_call(self,f,*args,**kwargs):
        logging.debug("Requesting call: %s(*%s,**%s)"%(f,args,kwargs))
        self.script_call_queue.put((f,args,kwargs))
    ##
    ##
    ##
    def ping_check(self,addresses,callback):
        fping_probe_socket=FPingProbeSocket(self.factory,self.config.get("path","fping"),addresses,callback)
    ##
    ##
    ##
    def check_event_source(self,address):
        return address in self.event_sources
        
    ##
    ## Main event loop
    ##
    def run(self):
        self.factory.run(run_forever=True)
    ##
    def tick(self):
        # Request filter updates
        if self.next_filter_update and time.time()>self.next_filter_update:
            self.get_event_filter()
        # Check for pending crashinfos
        if self.stand_alone_mode and self.next_crashinfo_check and time.time()>self.next_crashinfo_check:
            self.check_crashinfo()
        # Perform delayed calls
        while not self.script_call_queue.empty():
            try:
                f,args,kwargs=self.script_call_queue.get_nowait()
            except:
                break
            logging.debug("Calling delayed %s(*%s,**%s)"%(f,args,kwargs))
            apply(f,args,kwargs)
        # Perform default daemon/fsm machinery
        super(Activator,self).tick()
                
    def register_stream(self,stream):
        logging.debug("Registering stream %s"%str(stream))
        self.streams[stream]=None
        
    def release_stream(self,stream):
        logging.debug("Releasing stream %s"%str(stream))
        del self.streams[stream]
                
    def reboot(self):
        logging.info("Rebooting")
        os.execv(sys.executable,[sys.executable]+sys.argv)
        
    # Handlers
    ##
    ## Register
    ##
    @check_state("CONNECTED")
    def register(self):
        def register_callback(transaction,response=None,error=None):
            if self.get_state()!="CONNECTED":
                return
            if error:
                logging.error("Registration error: %s"%error.text)
                self.event("error")
                return
            logging.info("Registration accepted")
            self.nonce=response.nonce
            self.event("register")
        logging.info("Registering as '%s'"%self.config.get("activator","name"))
        r=RegisterRequest()
        r.name=self.activator_name
        self.sae_stream.proxy.register(r,register_callback)
    ##
    ## Auth
    ##
    @check_state("REGISTRED")
    def auth(self):
        def auth_callback(transaction,response=None,error=None):
            if self.get_state()!="REGISTRED":
                return
            if error:
                logging.error("Authentication failed: %s"%error.text)
                self.event("error")
                return
            logging.info("Authenticated")
            self.event("auth")
        logging.info("Authenticating")
        r=AuthRequest()
        r.name=self.config.get("activator","name")
        r.digest=get_digest(r.name,self.config.get("activator","secret"),self.nonce)
        self.sae_stream.proxy.auth(r,auth_callback)
        
    ##
    ##
    ##
    @check_state("UPGRADE")
    def manifest(self):
        def manifest_callback(transaction,response=None,error=None):
            if self.get_state()!="UPGRADE":
                return
            if error:
                logging.error("Manifest error: %s"%error.text)
                self.manifest_transaction=None
                return
            if transaction.id==self.manifest_transaction.id:
                update_list=[]
                for cs in response.files:
                    if not os.path.exists(cs.name) or cs.hash!=file_hash(cs.name):
                        update_list.append(cs.name)
                self.manifest_transaction=None
                if update_list:
                    self.software_upgrade(update_list)
                else:
                    self.event("establish")
            else:
                logging.error("Transaction id mismatch")
                self.manifest_transaction=None
        logging.info("Requesting manifest")
        r=ManifestRequest()
        self.manifest_transaction=self.sae_stream.proxy.manifest(r,manifest_callback)
    ##
    ## 
    ##
    @check_state("UPGRADE")
    def software_upgrade(self,update_list):
        def software_upgrade_callback(transaction,response=None,error=None):
            if error:
                logging.error("Upgrade error: %s"%error.text)
                self.software_upgrade_transaction=None
                return
            if transaction.id==self.software_upgrade_transaction.id:
                logging.info("Upgrading software")
                for u in response.codes:
                    logging.info("Upgrade: %s"%u.name)
                    safe_rewrite(u.name,u.code)
                self.software_upgrade_transaction=None
                self.reboot()
            else:
                logging.error("Transaction id mismatch")
                self.software_upgrade_transaction=None
        logging.debug("Requesting software upgrade for %s"%str(update_list))
        r=SoftwareUpgradeRequest()
        for f in update_list:
            r.names.append(f)
        self.software_upgrade_transaction=self.sae_stream.proxy.software_upgrade(r,software_upgrade_callback)
    ##
    ##
    ##
    @check_state("ESTABLISHED")
    def get_event_filter(self):
        def event_filter_callback(transaction,response=None,error=None):
            if error:
                logging.error("get_event_filter error: %s"%error.text)
                return
            self.event_sources=sets.Set(response.sources)
            self.next_filter_update=time.time()+response.expire
        r=EventFilterRequest()
        self.sae_stream.proxy.event_filter(r,event_filter_callback)
    ##
    ## Collect crashinfo files and send them as system events to SAE
    ## (Called only in standalone mode)
    ##
    @check_state("ESTABLISHED")
    def check_crashinfo(self):
        if not self.config.get("main","logfile"):
            return
        c_d=os.path.dirname(self.config.get("main","logfile"))
        if not os.path.isdir(c_d):
            return
        for fn in [fn for fn in os.listdir(c_d) if fn.startswith(DEBUG_CTX_CRASH_PREFIX)]:
            # Load and unpickle crashinfo
            path=os.path.join(c_d,fn)
            f=open(path)
            data=f.read()
            f.close()
            data=cPickle.loads(data)
            ts=data["ts"]
            del data["ts"]
            # Send event. "" is an virtual address of ROOT object
            self.on_event(ts,"",data)
            os.unlink(path)
        # Next check - after 60 seconds timeout
        self.next_crashinfo_check=time.time()+60
        
    ##
    ## Send FM event to SAE
    ##
    def on_event(self,timestamp,ip,body):
        def on_event_callback(transaction,response=None,error=None):
            if error:
                logging.error("event_proxy failed: %s"%error)
        r=EventRequest()
        r.timestamp=timestamp
        r.ip=ip
        for k,v in body.items():
            i=r.body.add()
            i.key=str(k)
            i.value=str(v)
        self.sae_stream.proxy.event(r,on_event_callback)
    # Signal handlers

    # SIGUSR1 returns process info
    def SIGUSR1(self,signo,frame):
        s=[
            ["factory.sockets",len(self.factory)],
        ]
        if self.sae_stream:
            s+=self.sae_stream.stats
        logging.info("STATS:")
        for n,v in s:
            logging.info("%s: %s"%(n,v))
    # SIGCHLD: Zombie hunting
    def SIGCHLD(self,signo,frame):
        while True:
            try:
                pid,status=os.waitpid(-1,os.WNOHANG)
            except:
                break
            if pid:
                logging.debug("Zombie pid=%d is hunted and mercilessly killed"%pid)
            else:
                break



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Finite State Machine
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import time,logging,re

##
## State checking decorator
##
def check_state(state):
    def check_returns(f):
        def new_f(self,*args, **kwds):
            assert self._current_state==state,\
                "Function '%s' cannot be called from state '%s' ('%s' required)"%(f.func_name,self._current_state,state)
            return f(self,*args, **kwds)
        new_f.func_name = f.func_name
        return new_f
    return check_returns

##
## Finite state machine class
##
class FSM(object):
    FSM_NAME="FSM"
    DEFAULT_STATE="DEFAULT" # Running state
    STATES={}               # STATE -> { EVENT -> New State}
    def __init__(self):
        self._current_state=None
        self._state_enter_time=None
        self._state_exit_time=None
        self.set_state(self.DEFAULT_STATE)

    def debug(self,msg):
        logging.debug("[%s(0x%x)]<%s> %s"%(self.__class__.__name__,id(self),self.get_state(),msg))

    def get_state_handler(self,state,event):
        name="on_%s_%s"%(state,event)
        try:
            return getattr(self,name)
        except:
            return None
            
    def call_state_handler(self,state,event,*args):
        h=self.get_state_handler(state,event)
        if h:
            apply(h,args)
            
    def get_state(self):
        return self._current_state
        
    def set_state(self,state):
        if state==self._current_state:
            return
        self.debug("==> %s"%state)
        if state not in self.STATES:
            raise Exception("Invalid state %s"%state)
        if self._current_state:
            self.call_state_handler(self._current_state,"exit")
        self._current_state=state
        self._state_enter_time=time.time()
        self._state_exit_time=None
        self.call_state_handler(state,"enter")
    
    def set_timeout(self,timeout):
        self.debug("set_timeout(%s)"%timeout)
        self._state_exit_time=time.time()+timeout
    ##
    ## Send event to FSM
    ##
    def event(self,event):
        self.debug("event(%s)"%event)
        if event not in self.STATES[self._current_state]:
            raise Exception("Invalid event '%s' in state '%s'"%(event,self._current_state))
        self.call_state_handler(self._current_state,event)
        self.set_state(self.STATES[self._current_state][event])
    ##
    ## Method should be called in event loop every second to process timeout and tick events
    ##
    def tick(self):
        if self._state_exit_time and self._state_exit_time<time.time():
            self.debug("Timeout expired")
            self.event("timeout")
        else:
            self.call_state_handler(self._current_state,"tick")
    
    ##
    ## Make nice GraphViz .dot chart
    ##
    @classmethod
    def get_dot(cls):
        r=["digraph {"]
        r+=["label=\"%s state machine\";"%cls.FSM_NAME]
        for s in cls.STATES:
            if s==cls.DEFAULT_STATE:
                r+=["%s [shape=\"doublecircle\"];"%s]
            else:
                r+=["%s;"%s]
            transforms={}
            for e,ns in cls.STATES[s].items():
                if ns in transforms:
                    transforms[ns].append(e)
                else:
                    transforms[ns]=[e]
            for ns,events in transforms.items():
                r+=["%s -> %s [label=\"%s\"];"%(s,ns,",\\n".join(events))]
        r+=["}",""]
        return "\n".join(r)
    @classmethod
    def write_dot(cls,path):
        f=open(path,"w")
        f.write(cls.get_dot())
        f.close()
##
## StreamFSM also changes state on input stream conditions
##
class StreamFSM(FSM):
    def __init__(self):
        self.patterns=[]
        self.in_buffer=""
        super(StreamFSM,self).__init__()
        
    def debug(self,msg):
        logging.debug("[%s(0x%x)]<%s> %s"%(self.__class__.__name__,id(self),self.get_state(),msg))
        
    def set_patterns(self,patterns):
        self.debug("set_patterns(%s)"%repr(patterns))
        self.patterns=[(re.compile(x,re.DOTALL|re.MULTILINE),y) for x,y in patterns]
        
    def feed(self,data,cleanup=None):
        self.debug("feed: %s"%repr(data))
        self.in_buffer+=data
        if cleanup:
            self.in_buffer=cleanup(self.in_buffer)
        while self.in_buffer and self.patterns:
            matched=False
            for rx,event in self.patterns:
                match=rx.search(self.in_buffer)
                if match:
                    matched=True
                    self.debug("match '%s'"%rx.pattern)
                    self.call_state_handler(self._current_state,"match",self.in_buffer[:match.start(0)],match)
                    self.in_buffer=self.in_buffer[match.end(0):]
                    self.event(event) # Change state
                    break
            if not matched:
                break



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## BGP events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
##
##
class BGPEstablished(EventClass):
    name = "BGP Established"
    category = "NETWORK"
    priority = "NORMAL"
    class Vars:
        neighbor_ip=Var(required=True)
##
## BGP Backward Transition
##
class BGPFSMStateChange(EventClass):
    name = "BGP FSM State Change"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="BGP Peer {{neighbor_ip}} state changed from {{from_state}} to {{to_state}}"
    body_template="""BGP Peer {{neighbor_ip}} changed state from {{from_state}} to {{to_state}}
    """
    class Vars:
        neighbor_ip=Var(required=True)
        from_state=Var(required=True)
        to_state=Var(required=True)
##
## BGP Backward Transition
##
class BGPBackwardTransition(EventClass):
    name = "BGP Backward Transition"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="BGP Peer backward transition: {{neighbor_ip}}"
    body_template="""BGP Peer {{neighbor_ip}} performs backward transition to state {{state}}
    """
    class Vars:
        neighbor_ip=Var(required=True)
        state=Var(required=True)
##
## BGP Max Prefixes Warning
##
class BGPMaxPrefixesWarning(EventClass):
    name     = "BGP Max Prefixes Warning"
    category = "NETWORK"
    priority = "WARNING"
    subject_template="BGP Prefixes warning: {{neighbor_ip}} ({{prefixes}}/{{max_prefixes}})"
    body_template="""Neighbor {{neighbor_ip}} reached {{prefixes}} prefixes with administrative limit of {{max_prefixes}}"""
    class Vars:
        neighbor_ip=Var(required=True)
        prefixes=Var(required=True)
        max_prefixes=Var(required=True)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.dns.models import *

RECORD_TYPES=[
    ("A",True),
    ("AAAA",True),
    ("AFSDB",False),
    ("AXFR",False),
    ("CERT",False),
    ("CNAME",True),
    ("DHCID",False),
    ("DLV",False),
    ("DNAME",False),
    ("DNSKEY",False),
    ("DS",False),
    ("HIP",False),
    ("IPSECKEY",False),
    ("IXFR",False),
    ("KEY",False),
    ("LOC",False),
    ("MX",True),
    ("NAPTR",True),
    ("NS",True),
    ("NSEC",False),
    ("NSEC3",False),
    ("NSEC3PARAM",False),
    ("OPT",False),
    ("PTR",True),
    ("RRSIG",False),
    ("SIG",False),
    ("SPF",True),
    ("SRV",True),
    ("SSHFP",False),
    ("TA",False),
    ("TKEY",False),
    ("TSIG",False),
    ("TXT",True),
    ]

class Migration:
    def forwards(self):
        for rtype,is_visible in RECORD_TYPES:
            if db.execute("SELECT COUNT(*) FROM dns_dnszonerecordtype WHERE type=%s",[rtype])[0][0]>0:
                continue
            print "Creating DNSZoneRecordType '%s'"%rtype
            db.execute("INSERT INTO dns_dnszonerecordtype(type,is_visible) VALUES(%s,%s)",[rtype,is_visible])
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Memory-related events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## Memory Allocation Error
##
class MemoryAllocationError(EventClass):
    name     = "Memory Allocation Error"
    category = "NETWORK"
    priority = "CRITICAL"
    subject_template="Memory allocation error"
    body_template="""Memory allocation error"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger=None



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Routing Policy Specification Language (RFC2622) routines
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------

## rpsl_format(rpsl)
## idents RPSL by aligning left side of values
## Emply strings and strings without ':' are silently dropped
RPLS_IDENT=20
def rpsl_format(rpsl,ident=None):
    if ident is None:
        ident=RPLS_IDENT
    out=[]
    mask="%%-%ds%%s"%ident
    for l in [x for x in rpsl.split("\n") if ":" in x]:
        k,v=l.split(":",1)
        out.append(mask%(k.strip()+":",v.strip()))
    return "\n".join(out)+"\n"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from django import forms
from models import *
##
class MIBDataInlineAdmin(admin.TabularInline):
    model=MIBData
    extra=0
##
class MIBAdmin(admin.ModelAdmin):
    list_display=["name","last_updated","uploaded"]
    search_fields=["name"]
    inlines=[MIBDataInlineAdmin]
##
class MIBDataAdmin(admin.ModelAdmin):
    list_display=["mib","oid","name"]
    search_fields=["oid","name"]
##
class EventPriorityAdmin(admin.ModelAdmin):
    list_display=["name","priority"]
    search_fields=["name"]
##
class EventCategoryAdmin(admin.ModelAdmin):
    list_display=["name"]
    search_fields=["name"]
##
class EventClassVarAdmin(admin.TabularInline):
    extra=1
    model=EventClassVar
##
class EventClassAdmin(admin.ModelAdmin):
    list_display=["name","category","default_priority","is_builtin","repeat_suppression","repeat_suppression_interval",
        "trigger","last_modified","python_link"]
    search_fields=["name"]
    list_filter=["is_builtin","category"]
    inlines=[EventClassVarAdmin]
##
class EventClassificationREAdmin(admin.TabularInline):
    extra=3
    model=EventClassificationRE
##
class EventClassificationRuleAdmin(admin.ModelAdmin):
    list_display=["name","event_class","preference","is_builtin","drop_event","python_link"]
    search_fields=["name"]
    list_filter=["is_builtin","drop_event","event_class"]
    inlines=[EventClassificationREAdmin]
##
class EventCorrelationRuleAdmin(admin.ModelAdmin):
    list_display=["name","is_builtin"]
    search_fields=["name"]
    list_filter=["is_builtin"]
##
admin.site.register(MIB, MIBAdmin)
admin.site.register(MIBData, MIBDataAdmin)
admin.site.register(EventPriority, EventPriorityAdmin)
admin.site.register(EventCategory, EventCategoryAdmin)
admin.site.register(EventClass, EventClassAdmin)
admin.site.register(EventClassificationRule, EventClassificationRuleAdmin)
admin.site.register(EventCorrelationRule, EventCorrelationRuleAdmin)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django's standard views module
## for MAIN module
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import django.contrib.auth
from django.http import HttpResponseRedirect,HttpResponseNotFound
from django.core.cache import cache
from noc.lib.render import render,render_success,render_failure,render_json
from noc.main.report import report_registry
from noc.main.menu import MENU
import os, types
##
## Startup boilerplate
##
def index(request):
    return render(request,"main/index.html")
##
## Log out current user
##
def logout(request):
    django.contrib.auth.logout(request)
    return HttpResponseRedirect("/")
##
## Called on 404 event
##
def handler404(request):
    return render(request,"main/404.html")
##
## Returns JSON with user's menu
##
def menu(request):
    cache_key="menu:%d"%request.user.id
    menu=cache.get(cache_key)
    if menu is None:
        menu=[]
        user=request.user
        for m in MENU:
            r=[]
            for mi in m["items"]:
                if len(mi)==3 and user.has_perm(mi[2]):
                    r+=[(mi[0],mi[1])]
                elif type(mi[1])==types.DictType:
                    sr=[(x[0],x[1]) for x in mi[1]["items"] if len(x)==2 or user.has_perm(x[2])]
                    if sr:
                        r+=[(mi[0],{"items":sr})]
            if r:
                mi={"items":r}
                for k,v in m.items():
                    if k=="items":
                        continue
                    mi[k]=v
                menu.append(mi)
        cache.set(cache_key,menu,300)
    return render_json(menu)
##
## Render report
##
def report(request,report):
    try:
        rc=report_registry[report]
    except KeyError:
        return HttpResponseNotFound("No report found")
    report=rc(request,request.POST)
    if report.is_valid():
        return report.render()
    else:
        return render(request,"main/report_form.html",{"report":report})
##
## Render report list
##
def report_index(request):
    r={}
    for cn,c in report_registry.classes.items():
        m,n=cn.split(".",1)
        if m not in r:
            r[m]=[c]
        else:
            r[m].append(c)
    out=[]
    keys=r.keys()
    keys.sort()
    for k in keys:
        v=r[k]
        v.sort(lambda x,y:cmp(x.title,y.title))
        out.append([k,v])
    return render(request,"main/report_index.html",{"reports":out})
##
## Success page
##
def success(request):
    print request.GET
    subject=request.GET.get("subject",None)
    text=request.GET.get("text",None)
    return render_success(request,subject=subject,text=text)
##
## Failure page
##
def failure(request):
    subject=request.GET.get("subject",None)
    text=request.GET.get("text",None)
    return render_failure(request,subject=subject,text=text)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report

class Report(noc.main.report.Report):
    name="dns.missed_p2p_addresses"
    title="/30 allocations without ip addresses"
    requires_cursor=True
    columns=[Column("Prefix",format=lambda x:"<A HREF='/ip/%s/'>%s</A>"%(x,x.split("/",1)[1]))]
    
    def get_queryset(self):
        vrf_id=self.execute("SELECT id FROM ip_vrf WHERE rd='0:0'")[0][0]
        return self.execute("""
            SELECT vrf_id||'/'||prefix
            FROM ip_ipv4block b
            WHERE masklen(prefix)=30
                AND vrf_id=%s
                AND (SELECT COUNT(*) FROM ip_ipv4address WHERE vrf_id=%s AND ip<<b.prefix)=0
            ORDER BY prefix""",[vrf_id,vrf_id])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Custom field types
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from django.db import models
from lib.ip import normalize_prefix
##
## CIDRField maps to PostgreSQL CIDR
##
class CIDRField(models.Field):
    def __init__(self,*args,**kwargs):
        super(CIDRField,self).__init__(*args,**kwargs)
        
    def db_type(self):
        return "CIDR"
        
    def to_python(self,value):
        return str(value)
        
    def get_db_prep_value(self,value):
        # Convert value to pure network address to prevent
        # PostgreSQL exception
        return normalize_prefix(value)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

NAMES=[
    "Other Normal",
    "Other Extended"
]

class Migration:
    
    def forwards(self):
        for n in NAMES:
            if db.execute("SELECT COUNT(*) FROM peer_communitytype WHERE name=%s",[n])[0][0]==0:
                db.execute("INSERT INTO peer_communitytype(name) VALUES(%s)",[n])
    
    def backwards(self):
        pass



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.cm.models import Object
from django.shortcuts import get_object_or_404
from noc.lib.render import render,render_plain_text
import os
from django.http import HttpResponseNotFound,HttpResponseRedirect,HttpResponseForbidden

def view(request,repo,object_id,revision=None,format="html"):
    o=get_object_or_404(Object.get_object_class(repo),id=int(object_id))
    if not o.has_access(request.user):
        return HttpResponseForbidden("Access denied")
    revs=o.revisions
    if revision:
        r=None
        for rev in revs:
            if rev.revision==revision:
                r=rev
                break
        if not r:
            return HttpResponseNotFound("Revision not found")
    else:
        if len(revs):
            r=o.revisions[0]
        else:
            r=None
    if r:
        content=o.get_revision(r)
    else:
        content=None
    if format=="html":
        return render(request,"cm/view.html",{"o":o,"r":r,"content":content})
    elif format=="text":
        return render_plain_text(content)
    else:
        return HttpResponseNotFound("Invalid format: %s"%format)

def diff(request,repo,object_id):
    o=get_object_or_404(Object.get_object_class(repo),id=int(object_id))
    if not o.has_access(request.user):
        return HttpResponseForbidden("Access denied")
    if request.POST and "r1" in request.POST and "r2" in request.POST:
        r1=o.find_revision(request.POST["r1"])
        r2=o.find_revision(request.POST["r2"])
        diff=o.diff(r1,r2)
        return render(request,"cm/diff.html",{"o":o,"diff":diff,"r1":r1,"r2":r2})
    else:
        return HttpResponseRedirect("/cm/view/config/%d/"%o.id)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Event triggers handlers
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.registry import Registry
import re
##
## Event Trigger Registry
## Performs triggers search and initialization
## inside fm/triggers/ directory
##
class EventTriggerRegistry(Registry):
    name="EventTriggerRegistry"
    subdir="triggers"
    classname="EventTrigger"
    apps=["noc.fm"]
event_trigger_registry=EventTriggerRegistry()

##
## Metaclass for EventTrigger.
## Performs trigger registration in Event Trigger Registry
##
class EventTriggerBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        event_trigger_registry.register(m.name,m)
        return m
##
## Event Trigger base class
##
class EventTrigger(object):
    __metaclass__=EventTriggerBase
    ## trigger name
    name=None
    ##
    ## Overload handle method to implement trigger functionality
    ## event is EventClass instance
    ##
    def handle(self,event):
        pass



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Version Control System support
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.registry import Registry
from lib.fileutils import copy_file
from noc.settings import config
import os,subprocess
##
## Registry for VCS
##
class VCSRegistry(Registry):
    name="VCSRegistry"
    subdir="vcs"
    classname="VCS"
    def get(self,repo):
        return self[config.get("cm","vcs_type")](repo)
vcs_registry=VCSRegistry()
##
##
##
class Revision(object):
    def __init__(self,revision,date):
        self.revision=revision
        self.date=date
    def __str__(self):
        return "%s (%s)"%(self.revision,self.date)
    def __repr__(self):
        return "<Revision %s (%s)>"%(self.revision,self.date)
        
##
## VCS Metaclass
##
class VCSBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        vcs_registry.register(m.name,m)
        return m
##
## VCS Base Class
##
class VCS(object):
    __metaclass__=VCSBase
    name=None
    option_commit_message="-m"
    def __init__(self,repo):
        self.repo=repo
    # Check wrether repository exists and create when necessary
    def check_repository(self):
        pass
    # Add file to repository
    def add(self,path):
        self.cmd(["add",path])
    # Remove file from repository
    def rm(self,path):
        self.cmd(["remove",path])
        self.commit(path,"rm")
    # Move file to a new location
    # Dumb emulation
    def mv(self,f,t):
        copy_file(os.path.join(self.repo,f),os.path.join(self.repo,t))
        self.add(t)
        self.rm(f)
        self.commit(t,"mv emulation")
    # Commit single file
    def commit(self,path,message="CM autocommit"):
        self.cmd(["commit",self.option_commit_message,message,path])
    #
    # Execute VCS command.
    # cmd is a list of arguments
    def cmd(self,cmd,check=True):
        if check:
            self.check_repository()
        subprocess.check_call([config.get("cm","vcs_path")]+cmd,cwd=self.repo)
    # Returns an output of cmd
    def cmd_out(self,cmd):
        p=subprocess.Popen([config.get("cm","vcs_path")]+cmd,stdout=subprocess.PIPE,cwd=self.repo)
        d=p.stdout.read()
        return d
    # Returns a list of Revision
    def log(self,path):
        raise Exception("Not supported")
    # Returns unified diff
    def diff(self,path,rev1,rev2):
        raise Exception("Not supported")
    # Returns revision of the file
    def get_revision(self,path,revision):
        raise Exception("Not supported")
    #
    def in_repo(self,path):
        return os.path.exists(os.path.join(self.repo,path))



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## LDP events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
##
##
#class LDPSessionUp(EventClass): pass
##
##
##
#class LDPSessionDown(EventClass): pass
##
##
##
class LDPLSPUp(EventClass):
    name="LDP LSP Up"
    category="NETWORK"
    priority="NORMAL"
    subject_template="LDP LSP Up: {{fec}}"
    body_template="""LDP LSP Up: {{fec}}"""
    class Vars:
        router_id=Var(required=True)
        fec=Var(required=True)
##
##
##
class LDPLSPDown(EventClass):
    name="LDP LSP Down"
    category="NETWORK"
    priority="MAJOR"
    subject_template="LDP LSP Down: {{fec}}"
    body_template="""LDP LSP Down: {{fec}}"""
    class Vars:
        router_id=Var(required=True)
        fec=Var(required=True)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco IOS specific EIGRP rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.eigrp import *
##
## Cisco.IOS EIGRP Neighbor Up SYSLOG
##
class Cisco_IOS_EIGRP_Neighbor_Up_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS EIGRP Neighbor Up SYSLOG"
    event_class=EIGRPNeighborUp
    preference=1000
    patterns=[
        (r"^source$",r"^syslog$"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^message$",r"%DUAL-5-NBRCHANGE: IP-EIGRP\(\d+\) \d+: Neighbor (?P<ip>\S+) \((?P<interface>\S+)\) is up"),
    ]
##
## Cisco.IOS EIGRP Neighbor Down SYSLOG
##
class Cisco_IOS_EIGRP_Neighbor_Down_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS EIGRP Neighbor Down SYSLOG"
    event_class=EIGRPNeighborDown
    preference=1000
    patterns=[
        (r"^source$",r"^syslog$"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^message$",r"%DUAL-5-NBRCHANGE: IP-EIGRP\(\d+\) \d+: Neighbor (?P<ip>\S+) \((?P<interface>\S+)\) is down"),
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.daemon import Daemon
from django.core.servers.fastcgi import runfastcgi

class FCGI(Daemon):
    daemon_name="noc-fcgi"
    def run(self):
        runfastcgi(["method=threaded", 
                    "daemonize=false",
                    "socket=%s"%self.config.get("fcgi","socket"),
                    "minspare=%d"%self.config.getint("fcgi","minspare"),
                    "maxspare=%d"%self.config.getint("fcgi","maxspare"),
                    "maxrequests=%s"%self.config.getint("fcgi","maxrequests"),
                    "maxchildren=%s"%self.config.getint("fcgi","maxchildren")])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## noc-classifier daemon
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.daemon import Daemon
from noc.lib.pyquote import bin_quote,bin_unquote
from noc.lib.validators import is_ipv4
from noc.fm.models import EventClassificationRule,Event,EventData,EventClass,MIB,EventClassVar,EventRepeat
from django.db import transaction
import re,logging,time,datetime

##
## Patterns
##
rx_template=re.compile(r"\{\{([^}]+)\}\}")
rx_oid=re.compile(r"^(\d+\.){6,}")
rx_mac_cisco=re.compile(r"^[0-9a-f]{4}\.[0-9a-f]{4}\.[0-9a-f]{4}$")
##
## Exceptions
##
class DecodeError(Exception): pass
##
## In-memory Rule representation
##
class Rule(object):
    def __init__(self,rule):
        self.rule=rule
        self.name=rule.name
        self.re=[(re.compile(x.left_re,re.MULTILINE|re.DOTALL),re.compile(x.right_re,re.MULTILINE|re.DOTALL)) for x in rule.eventclassificationre_set.all()]
    ##
    ## Return a hash of extracted variables for object o, or None
    ##
    def match(self,o):
        vars={}
        for l,r in self.re:
            found=False
            for o_l,o_r in o:
                l_match=l.search(o_l)
                if not l_match:
                    continue
                r_match=r.search(o_r)
                if not r_match:
                    return None
                found=True
                # Apply decoders if necessary
                # Decoders are given as (?P<name__decoder>.....) patters
                try:
                    for gd in [l_match.groupdict(),r_match.groupdict()]:
                        for k,v in gd.items():
                            if "__" in k:
                                k_name,decoder=k.split("__",1)
                                vars[k_name]=getattr(self,"decode_%s"%decoder)(v) # Apply decoder
                            else:
                                vars[k]=v # Pass unchanged
                except DecodeError:
                    return None # No match when decoder failed
                except AttributeError:
                    return None # No match when decoder not found
                break
            if not found:
                return None
        return vars
    ## Rule is drop rule
    def _drop_event(self):
        return self.rule.drop_event
    drop_event=property(_drop_event)
    ##
    ## Decode IPv4 from 4 octets
    ##
    def decode_ipv4(self,s):
        if len(s)==4:
            return ".".join(["%d"%ord(c) for c in list(s)])
        if is_ipv4(s):
            return s
        raise DecodeError
    ##
    ## Decode MAC address from 6 octets
    ##
    def decode_mac(self,s):
        if len(s)==6:
            print repr(s)
            return "%02X:%02X:%02X:%02X:%02X:%02X"%tuple([ord(x) for x in list(s)])
        if rx_mac_cisco.match(s):
            s=s.replace(".","").upper()
            return "%s:%s:%s:%s:%s:%s"%(s[:2],s[2:4],s[4:6],s[6:8],s[8:10],s[10:])
        raise DecodeError
##
## Noc-classifier daemon
##
class Classifier(Daemon):
    daemon_name="noc-classifier"
    def __init__(self):
        self.rules=[]
        Daemon.__init__(self)
        logging.info("Running Classifier")
    ##
    ## Load rules from database after loading config
    ##
    def load_config(self):
        super(Classifier,self).load_config()
        self.load_rules()
    ##
    ## Load rules from database
    ##
    def load_rules(self):
        logging.info("Loading rules")
        self.rules=[Rule(r) for r in EventClassificationRule.objects.order_by("preference")]
        logging.info("%d rules loaded"%len(self.rules))
    ##
    ## Replace all variable occurences in template by variables content
    ##
    def expand_template(self,template,vars):
        return rx_template.sub(lambda m:str(vars.get(m.group(1),"{{UNKNOWN VAR}}")),template)
    ##
    ## Classify single event:
    ## 1. Resolve OIDs when source is SNMP Trap
    ## 2. Try to find matching rule
    ## 3. Drop event if required by rule
    ## 4. Set event class of the matched rule or DEFAULT
    ## 
    def classify_event(self,cursor,event):
        def is_oid(s):
            return rx_oid.search(s) is not None
        # Extract received event properties
        props=[(x.key,bin_unquote(x.value)) for x in event.eventdata_set.filter(type=">")]
        # Resolve additional event properties
        source=None
        for k,v in props:
            if k=="source":
                source=v
                break
        resolved={
            "profile":event.managed_object.profile_name
        }
        # Resolve SNMP oids
        if source=="SNMP Trap":
            for k,v in props:
                if is_oid(k):
                    oid=MIB.get_name(k)
                    if oid!=k:
                        if is_oid(v):
                            v=MIB.get_name(v)
                        resolved[oid]=v
        if resolved:
            props+=resolved.items()
        # Find rule
        event_class=None
        # Try to find matching rule
        rule_id=0
        for r in self.rules:
            # Try to match rule
            vars=r.match(props)
            if vars is None:
                continue
            rule_id=r.rule.id
            # Silently drop event when required by rule
            if r.drop_event:
                logging.debug("Drop event %d"%event.id)
                [ed.delete() for ed in event.eventdata_set.all()]
                event.delete()
                return
            event_class=r.rule.event_class
            logging.debug("Matching class for event %d found: %s (Rule: %s)"%(event.id,event_class.name,r.name))
            # Check the event is repeatition of existing one
            if event_class.repeat_suppression and event_class.repeat_suppression_interval>0:
                # Delete event as repeatition of the known event
                # Build keys
                kv={}
                for name in [v.name for v in EventClassVar.objects.filter(event_class=event_class,repeat_suppression=True)]:
                    if name in vars:
                        kv[name]=vars[name]
                    else:
                        kv=None
                        break
                if kv is not None:
                    r=[e for e in Event.objects.filter(
                        event_class=event_class,
                        managed_object=event.managed_object,
                        timestamp__gte=event.timestamp-datetime.timedelta(seconds=event_class.repeat_suppression_interval),
                        timestamp__lte=event.timestamp
                        ).exclude(id=event.id).order_by("-timestamp")
                        if e.match_data(kv)]
                    if len(r)>0:
                        pe=r[0]
                        logging.debug("Event #%d repeats event #%d"%(event.id,pe.id))
                        er=EventRepeat(event=pe,timestamp=pe.timestamp)
                        er.save()
                        pe.timestamp=event.timestamp
                        pe.save()
                        [ed.delete() for ed in event.eventdata_set.all()]
                        event.delete()
                        return
            break
        # Set event class to DEFAULT when no matching rule found
        if event_class is None:
            event_class=EventClass.objects.get(name="DEFAULT")
            vars={}
            logging.debug("No rule found for event %d. Falling back to DEFAULT"%event.id)
        # Fill event subject and body
        f_vars=dict(props) # f_vars contains all event vars, including original, extracted and resolved
        f_vars.update(vars)
        f_vars.update(resolved)
        subject=self.expand_template(event_class.subject_template,f_vars)
        if len(subject)>255: # Too long subject must be truncated
            subject=subject[:250]+" ..."
        body=self.expand_template(event_class.body_template,f_vars)
        # Prepare and call update_event_classification stored procedure for bulk event update
        v_args=[]
        for k,v in resolved.items():
            v_args+=["R",k,bin_quote(v)]
        for k,v in vars.items():
            v_args+=["V",k,bin_quote(v)]
        p="SELECT update_event_classification(%s,%s,%s,%s,%s,%s,%s,ARRAY["
        p+=",".join(["ARRAY[%s,%s,%s]"]*(len(v_args)/3))
        p+="])"
        cursor.execute(p,[event.id,rule_id,event_class.id,event_class.category.id,event_class.default_priority.id,subject,body]+v_args)
        cursor.execute("COMMIT")
        # Finally run event class trigger
        event.event_class.run_trigger(event)
        
    def run(self):
        INTERVAL=10
        last_sleep=time.time()
        transaction.enter_transaction_management()
        from django.db import connection
        cursor = connection.cursor()
        while True:
            n=0
            t0=time.time()
            for e in Event.objects.filter(status="U").order_by("id"):
                self.classify_event(cursor,e)
                transaction.commit()
                n+=1
            if n:
                dt=time.time()-t0
                if dt>0:
                    perf=n/dt
                else:
                    perf=0
                logging.info("%d events classified (%10.4f second elapsed. %10.4f events/sec)"%(n,dt,perf))
            t=time.time()
            if t-last_sleep<=INTERVAL:
                time.sleep(INTERVAL-t+last_sleep)
            last_sleep=t
        transaction.leave_transaction_management()
        




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.core.management.base import BaseCommand
from django.db import transaction
from noc.fm.models import MIB

class Command(BaseCommand):
    help="Import MIB into database"
    def handle(self, *args, **options):
        transaction.enter_transaction_management()
        for a in args:
            MIB.load(a)
        transaction.commit()
        transaction.leave_transaction_management()

        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.periodic
from noc.settings import config
import os,subprocess,datetime
import logging

class Task(noc.sa.periodic.Task):
    name="main.cleanup_sessions"
    description=""
    
    def execute(self):
        from django.db import connection
        cursor=connection.cursor()
        cursor.execute("DELETE FROM django_session WHERE expire_date<'now'")
        cursor.execute("COMMIT")
        return True



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Service Activation Engine Daemon
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from __future__ import with_statement
from noc.sa.models import Activator, ManagedObject, TaskSchedule
from noc.fm.models import Event,EventData,EventPriority,EventClass,EventCategory
from noc.sa.rpc import RPCSocket,file_hash,get_digest,get_nonce
import logging,time,threading,datetime,os,sets,random,xmlrpclib,cPickle
from noc.sa.protocols.sae_pb2 import *
from noc.lib.fileutils import read_file
from noc.lib.daemon import Daemon
from noc.lib.debug import error_report,DEBUG_CTX_CRASH_PREFIX
from noc.lib.nbsocket import ListenTCPSocket,AcceptedTCPSocket,SocketFactory,Protocol

##
## Additions to MANIFEST-ACTIVATOR file
##
ACTIVATOR_MANIFEST=[
    "sa/profiles/",
    "sa/interfaces/",
]

##
##
##
class Service(SAEService):
    def get_controller_activator(self,controller):
        return Activator.objects.get(name=self.sae.factory.get_name_by_socket(controller.stream))
    ##
    ## RPC interfaces
    ##
    def ping(self,controller,request,done):
        done(controller,response=PingResponse())
    
    def register(self,controller,request,done):
        try:
            activator=Activator.objects.get(name=request.name)
        except Activator.DoesNotExist:
            logging.error("Unknown activator '%s'"%request.name)
            e=Error()
            e.code=ERR_UNKNOWN_ACTIVATOR
            e.text="Unknown activator '%s'"%request.name
            done(controller,error=e)
            return
        logging.info("Requesting digest for activator '%s'"%request.name)
        r=RegisterResponse()
        r.nonce=get_nonce()
        controller.stream.nonce=r.nonce
        done(controller,response=r)
        
    def auth(self,controller,request,done):
        try:
            activator=Activator.objects.get(name=request.name)
        except Activator.DoesNotExist:
            logging.error("Unknown activator '%s'"%request.name)
            e=Error()
            e.code=ERR_UNKNOWN_ACTIVATOR
            e.text="Unknown activator '%s'"%request.name
            done(controller,error=e)
            return
        logging.info("Authenticating activator '%s'"%request.name)
        if controller.stream.nonce is None or get_digest(request.name,activator.auth,controller.stream.nonce)!=request.digest:
            e=Error()
            e.code=ERR_AUTH_FAILED
            e.text="Authencication failed for activator '%s'"%request.name
            done(controller,error=e)
            return
        r=AuthResponse()
        controller.stream.set_name(request.name)
        controller.stream.is_authenticated=True
        done(controller,response=r)
        
    def manifest(self,controller,request,done):
        if not controller.stream.is_authenticated:
            e=Error()
            e.code=ERR_AUTH_REQUIRED
            e.text="Authentication required"
            done(controller,error=e)
            return
        done(controller,response=self.sae.activator_manifest)
        
    def software_upgrade(self,controller,request,done):
        if not controller.stream.is_authenticated:
            e=Error()
            e.code=ERR_AUTH_REQUIRED
            e.text="Authentication required"
            done(controller,error=e)
            return
        r=SoftwareUpgradeResponse()
        for n in request.names:
            if n not in self.sae.activator_manifest_files:
                e=Error()
                e.code=ERR_INVALID_UPGRADE
                e.text="Invalid file requested for upgrade: %s"%n
                done(controller,error=e)
                return
            u=r.codes.add()
            u.name=n
            u.code=read_file(n)
        done(controller,response=r)
        
    def event_filter(self,controller,request,done):
        if not controller.stream.is_authenticated:
            e=Error()
            e.code=ERR_AUTH_REQUIRED
            e.text="Authentication required"
            done(controller,error=e)
            return
        activator=self.get_controller_activator(controller)
        r=EventFilterResponse()
        r.expire=self.sae.config.getint("sae","refresh_event_filter")
        for c in ManagedObject.objects.filter(activator=activator,trap_source_ip__isnull=False):
            r.sources.append(c.trap_source_ip)
        done(controller,response=r)
        
    def event(self,controller,request,done):
        if not controller.stream.is_authenticated:
            e=Error()
            e.code=ERR_AUTH_REQUIRED
            e.text="Authentication required"
            done(controller,error=e)
            return
        activator=self.get_controller_activator(controller)
        try:
            if request.ip=="":
                # Event belongs to ROOT object
                mo=ManagedObject.objects.get(name="ROOT")
            else:
                mo=ManagedObject.objects.get(activator=activator,trap_source_ip=request.ip)
        except ManagedObject.DoesNotExist:
            e=Error()
            e.code=ERR_UNKNOWN_EVENT_SOURCE
            e.text="Unknown event source '%s'"%request.ip
            done(controller,error=e)
            return
        # Do all the magic here
        e=Event(
            timestamp=datetime.datetime.fromtimestamp(request.timestamp),
            event_priority=EventPriority.objects.get(name="DEFAULT"),
            event_class=EventClass.objects.get(name="DEFAULT"),
            event_category=EventCategory.objects.get(name="DEFAULT"),
            managed_object=mo
            )
        e.save()
        for b in request.body:
            d=EventData(event=e,key=b.key,value=b.value)
            d.save()
        done(controller,EventResponse())
    
#    def event_config_changed(self,controller,request,done):
#        if not controller.stream.is_authenticated:
#            e=Error()
#            e.code=ERR_AUTH_REQUIRED
#            e.text="Authentication required"
#            done(controller,error=e)
#            return
#        activator=self.get_controller_activator(controller)
#        try:
#            c=ManagedObject.objects.get(activator=activator,trap_source_ip=request.ip)
#        except ManagedObject.DoesNotExist:
#            e=Error()
#            e.code=ERR_UNKNOWN_EVENT_SOURCE
#            e.text="Unknown event source '%s'"%request.ip
#            done(controller,error=e)
#            return
#        logging.info("%s configuration changed"%(c.repo_path))
#        c.next_pull=min(c.next_pull,datetime.datetime.now()+datetime.timedelta(minutes=10))
#        c.save()
#        done(controller,EventResponse())

##
## AcceptedTCPSocket with RPC Protocol
##
class SAESocket(RPCSocket,AcceptedTCPSocket):
    def __init__(self,factory,socket):
        AcceptedTCPSocket.__init__(self,factory,socket)
        RPCSocket.__init__(self,factory.sae.service)
        self.nonce=None
        self.is_authenticated=True
        
    @classmethod
    def check_access(cls,address):
        return Activator.objects.filter(ip=address).count()>0
##
## XML-RPC support
##

##
## Service
##
class XMLRPCService(object):
    def __init__(self,sae):
        self._sae=sae
        
    def _call_xmlrpc_method(self,done,method,*args):
        getattr(self,method)(done,*args)
        
    def listMethods(self,done):
        done([m for m in dir(self) if not m.startswith("_") and callable(getattr(self,m))])
    
    def script(self,done,name,object_id,kwargs):
        logging.info("XML-RPC.script %s(object_id=%d)"%(name,int(object_id)))
        object=ManagedObject.objects.get(id=int(object_id))
        self._sae.script(object,name,done,**kwargs)

##
## PDU Parsing
##
class XMLRPCProtocol(Protocol):
    def parse_pdu(self):
        r=[]
        while True:
            idx=self.in_buffer.find("</methodCall>")
            if idx==-1:
                break
            r.append(self.in_buffer[:idx+13])
            self.in_buffer=self.in_buffer[idx+13:]
        return r
##
## N.B. Socket and XML-RPC HTTP server
##
class XMLRPCSocket(AcceptedTCPSocket):
    protocol_class=XMLRPCProtocol
    def __init__(self,factory,socket):
        AcceptedTCPSocket.__init__(self,factory,socket)
        
    def on_read(self,data):
        def on_read_callback(result=None,error=None):
            if error:
                body=xmlrpclib.dumps(xmlrpclib.Fault(ERR_SCRIPT_EXCEPTION,"Script exception"),methodresponse=True)
            else:
                body=xmlrpclib.dumps((result,),methodresponse=True)
            response="HTTP/1.1 200 OK\r\nContent-Type: text/xml\r\nContent-Length: %d\r\nServer: nocproject.org\r\n\r\n"%len(body)+body
            self.write(response)
            self.close(flush=True)
        headers,data=data.split("\r\n\r\n")
        params,methodname=xmlrpclib.loads(data,use_datetime=True)
        self.debug("XMLRPC Call: %s(*%s)"%(methodname,params))
        self.factory.sae.xmlrpc_service._call_xmlrpc_method(on_read_callback,methodname,*params)


##
## SAE Supervisor
##
class SAE(Daemon):
    daemon_name="noc-sae"
    def __init__(self):
        Daemon.__init__(self)
        logging.info("Running SAE")
        #
        self.service=Service()
        self.service.sae=self
        #
        self.xmlrpc_service=XMLRPCService(self)
        #
        self.factory=SocketFactory()
        self.factory.sae=self
        #
        self.sae_listener=None
        self.xmlrpc_listener=None
        # Periodic tasks
        self.active_periodic_tasks={}
        self.periodic_task_lock=threading.Lock()
        #
        self.activator_manifest=None
        self.activator_manifest_files=None
    
    def build_manifest(self):
        logging.info("Building manifest")
        manifest=read_file("MANIFEST-ACTIVATOR").split("\n")+ACTIVATOR_MANIFEST
        manifest=[x.strip() for x in manifest if x]
        self.activator_manifest=ManifestResponse()
        self.activator_manifest_files=sets.Set()
        
        files=sets.Set()
        for f in manifest:
            if f.endswith("/"):
                for dirpath,dirnames,filenames in os.walk(f):
                    for f in [f for f in filenames if f.endswith(".py")]:
                        files.add(os.path.join(dirpath,f))
            else:
                files.add(f)
        for f in files:
            self.activator_manifest_files.add(f)
            cs=self.activator_manifest.files.add()
            cs.name=f
            cs.hash=file_hash(f)
            
    def start_listeners(self):
        # SAE Listener
        sae_listen=self.config.get("sae","listen")
        sae_port=self.config.getint("sae","port")
        if self.sae_listener and (self.sae_listener.address!=sae_listen or self.sae_listener.port!=sae_port):
            self.sae_listener.close()
            self.sae_listener=None
        if self.sae_listener is None:
            logging.info("Starting SAE listener at %s:%d"%(sae_listen,sae_port))
            self.sae_listener=self.factory.listen_tcp(sae_listen,sae_port,SAESocket)
        # XML-RPC listener
        xmlrpc_listen=self.config.get("xmlrpc","listen")
        xmlrpc_port=self.config.getint("xmlrpc","port")
        if self.xmlrpc_listener and (self.xmlrpc_listener.address!=xmlrpc_listen or self.xmlrpc_listen.port!=xmlrpc_port):
            self.xmlrpc_listener.close()
            self.xmlrpc_listener=None
        if self.xmlrpc_listener is None:
            logging.info("Starting XML-RPC listener at %s:%d"%(xmlrpc_listen,xmlrpc_port))
            self.sae_listener=self.factory.listen_tcp(xmlrpc_listen,xmlrpc_port,XMLRPCSocket)
        
    def run(self):
        self.build_manifest()
        self.start_listeners()
        last_cleanup=time.time()
        last_task_check=time.time()
        last_crashinfo_check=time.time()
        while True:
            self.factory.loop(1)
            if time.time()-last_task_check>=10:
                self.periodic_task_lock.acquire()
                tasks = TaskSchedule.get_pending_tasks(exclude=self.active_periodic_tasks.keys())
                self.periodic_task_lock.release()
                last_task_check=time.time()
                if tasks:
                    for t in tasks:
                        self.run_periodic_task(t)
            if time.time()-last_crashinfo_check>=60:
                self.collect_crashinfo()
                last_crashinfo_check=time.time()
    ##
    ## Collect crashinfo and write as FM events
    ##
    def collect_crashinfo(self):
        if not self.config.get("main","logfile"):
            return
        c_d=os.path.dirname(self.config.get("main","logfile"))
        if not os.path.isdir(c_d):
            return
        mo=ManagedObject.objects.get(name="ROOT")
        event_priority=EventPriority.objects.get(name="DEFAULT")
        event_class=EventClass.objects.get(name="DEFAULT")
        event_category=EventCategory.objects.get(name="DEFAULT")
        for fn in [fn for fn in os.listdir(c_d) if fn.startswith(DEBUG_CTX_CRASH_PREFIX)]:
            path=os.path.join(c_d,fn)
            try:
                with open(path,"r") as f:
                    data=cPickle.loads(f.read())
            except:
                logging.error("Cannot import crashinfo: %s"%path)
                continue
            ts=data["ts"]
            del data["ts"]
            e=Event(
                timestamp=datetime.datetime.fromtimestamp(ts),
                event_priority=event_priority,
                event_class=event_class,
                event_category=event_category,
                managed_object=mo
            )
            e.save()
            for k,v in data.items():
                d=EventData(event=e,key=k,value=v)
                d.save()
            os.unlink(path)
            
    ##
    ## Periodic tasks
    ##
    def run_periodic_task(self,task):
        logging.debug(u"New task running: %s"%unicode(task))
        t=threading.Thread(name=task.periodic_name,target=self.periodic_wrapper,kwargs={"task":task})
        self.periodic_task_lock.acquire()
        self.active_periodic_tasks[task.id]=t
        self.periodic_task_lock.release()
        t.start()
    
    def periodic_wrapper(self,task):
        logging.info(u"Executing %s"%unicode(task))
        cwd=os.getcwd()
        try:
            status=task.periodic_class(self).execute()
        except:
            error_report()
            status=False
        logging.info(u"Task %s is terminated with '%s'"%(unicode(task),status))
        if status:
            timeout=task.run_every
        else:
            timeout=max(60,task.run_every/4)
        task.next_run=datetime.datetime.now()+datetime.timedelta(seconds=timeout)
        task.save()
        self.periodic_task_lock.acquire()
        try:
            del self.active_periodic_tasks[task.id]
        finally:
            self.periodic_task_lock.release()
        new_cwd=os.getcwd()
        if cwd!=new_cwd:
            logging.error("CWD changed by periodic '%s' ('%s' -> '%s'). Restoring old cwd"%(unicode(task),cwd,new_cwd))
            os.chdir(cwd)
        
    def on_stream_close(self,stream):
        self.streams.unregister(stream)
        
    def register_activator(self,name,stream):
        self.streams.register(stream,name)
        
    def get_activator_stream(self,name):
        try:
            return self.factory.get_socket_by_name(name)
        except KeyError:
            raise Exception("Activator not available: %s"%name)
    
    def script(self,object,name,callback,**kwargs):
        def script_callback(transaction,response=None,error=None):
            if error:
                logging.error("script(%s,**%s) failed: %s"%(name,kwargs,error.text))
                callback(error=error)
                return
            result=response.result
            result=cPickle.loads(str(result)) # De-serialize
            callback(result=result)
        logging.info("script %s(%s)"%(name,object))
        try:
            stream=self.get_activator_stream(object.activator.name)
        except:
            e=Error()
            e.code=ERR_ACTIVATOR_NOT_AVAILABLE
            e.text="Activator '%s' not available"%object.activator.name
            logging.error(e.text)
            callback(error=e)
            return
        r=ScriptRequest()
        r.script=name
        r.access_profile.profile           = object.profile_name
        r.access_profile.scheme            = object.scheme
        r.access_profile.address           = object.address
        if object.port:
            r.access_profile.port          = object.port
        if object.user:
            r.access_profile.user          = object.user
        if object.password:
            r.access_profile.password      = object.password
        if object.super_password:
            r.access_profile.super_password= object.super_password
        if object.remote_path:
            r.access_profile.path          = object.remote_path
        if object.snmp_ro:
            r.access_profile.snmp_ro       = object.snmp_ro
        if object.snmp_rw:
            r.access_profile.snmp_rw       = object.snmp_rw
        for k,v in kwargs.items():
            a=r.kwargs.add()
            a.key=str(k)
            a.value=str(v)
        stream.proxy.script(r,script_callback)
    ##
    ## Send a list of addresses to activator
    ## and generate fault events for unreachable ones
    ##
    def ping_check(self,activator,addresses):
        def ping_check_callback(transaction,response=None,error=None):
            def save_probe_result(u,result):
                try:
                    mo=ManagedObject.objects.get(activator=activator,trap_source_ip=u)
                except ManagedObject.DoesNotExist:
                    logging.error("Unknown object in ping_check: %s"%u)
                    return
                e=Event(
                    timestamp=ts,
                    event_priority=event_priority,
                    event_class=event_class,
                    event_category=event_category,
                    managed_object=mo
                    )
                e.save()
                for k,v in [("source","system"),
                            ("activator",activator_name),
                            ("probe","ping"),
                            ("ip",u),
                            ("result",result)]:
                    d=EventData(event=e,key=k,value=v)
                    d.save()
            if error:
                logging.error("ping_check failed: %s"%error.text)
                return
            ts=datetime.datetime.now()
            event_priority=EventPriority.objects.get(name="DEFAULT")
            event_class=EventClass.objects.get(name="DEFAULT")
            event_category=EventCategory.objects.get(name="DEFAULT")
            activator_name=activator.name
            for u in response.unreachable:
                save_probe_result(u,"failed")
            for u in response.reachable:
                save_probe_result(u,"success")
        logging.debug("ping_check(%s)"%activator.name)
        try:
            stream=self.get_activator_stream(activator.name)
        except:
            e=Error()
            e.code=ERR_ACTIVATOR_NOT_AVAILABLE
            e.text="Activator '%s' not available"%activator.name
            logging.error(e.text)
            return
        r=PingCheckRequest()
        for a in addresses:
            r.addresses.append(a)
        stream.proxy.ping_check(r,ping_check_callback)
    ##
    ## Called after config reloaded by SIGHUP.
    ##
    def on_load_config(self):
        self.start_listeners()
    # Signal handlers

    # SIGUSR1 returns process info
    def SIGUSR1(self,signo,frame):
        s=[
            ["factory.sockets",len(self.factory)],
        ]
        logging.info("STATS:")
        for n,v in s:
            logging.info("%s: %s"%(n,v))
        for sock in [s for s in self.factory.sockets.values() if issubclass(s.__class__,RPCSocket)]:
            try:
                logging.info("Activator: %s"%self.factory.get_name_by_socket(sock))
            except KeyError:
                logging.info("Unregistred activator")
            for n,v in sock.stats:
                logging.info("%s: %s"%(n,v))



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Load and syncronized built-in FM classes, rules and MIBs
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from __future__ import with_statement
from django.core.management.base import BaseCommand
from django.db import transaction
from noc.fm.rules.classes import EventClass
from noc.fm.rules.classification import ClassificationRule
from noc.fm.models import MIB,MIBRequiredException,EventClassificationRule,EventCorrelationRule
import os,re
##
## patterns
##
rx_comment=re.compile(r"#+(.*)\s*$")
##
## Command handler
##
class Command(BaseCommand):
    help="Syncronize built-in FM classes and rules"
    def handle(self, *args, **options):
        transaction.enter_transaction_management()
        self.sync_mibs()
        self.sync_classes()
        self.sync_classification_rules()
        self.sync_correlation_rules()
        transaction.leave_transaction_management()
    ##
    ## Search for subclasses of givent class inside given directory
    ##
    def search(self,cls,d):
        classes={}
        for dirpath,dirnames,filenames in os.walk(d):
            mb="noc.fm."+".".join(dirpath.split(os.sep)[1:])+"."
            for f in [f for f in filenames if f.endswith(".py") and f!="__init__.py"]:
                m=__import__(mb+f[:-3],{},{},"*")
                for ec_name in dir(m):
                    ec=getattr(m,ec_name)
                    try:
                        if not issubclass(ec,cls) or ec==cls:
                            continue
                    except:
                        continue
                    classes[ec]=None
        return classes.keys()
    ##
    ## Syncronize event classes
    ##
    def sync_classes(self):
        for c in self.search(EventClass,"fm/rules/classes"):
            c.sync()
    ##
    ## Syncronize event classification rules
    ##
    def sync_classification_rules(self):
        # Make built-in rules list
        loaded_rules={}
        for r in EventClassificationRule.objects.filter(is_builtin=True):
            loaded_rules[r.name]=None
        # Sync new rules
        for c in self.search(ClassificationRule,"fm/rules/classification"):
            c.sync()
            if c.name in loaded_rules:
                del loaded_rules[c.name]
        # Delete stale rules
        for r in loaded_rules:
            rule=EventClassificationRule.objects.get(name=r)
            rule.delete()
            print "DELETE CLASSIFICATION RULE %s"%r
    ##
    ## Syncronize event correlation rules
    ##
    def sync_correlation_rules(self):
        # Generator returning rules in file
        def get_rules(path):
            pkg="_".join(path.split(os.path.sep)[3:])[:-4]
            with open(path) as f:
                description=[]
                body=[]
                name=None
                for l in f.readlines():
                    l=l[:-1].replace("\t","    ")
                    if l.startswith(" "):
                        body+=[l[4:]]
                        continue
                    else:
                        if body and name:
                            yield {
                                "name"        : name,
                                "body"        : "\n".join(body),
                                "description" : "\n".join(description)
                            }
                            name=None
                            body=[]
                            description=[]
                        if l.startswith("#"):
                            match=rx_comment.match(l)
                            if match:
                                description+=[match.group(1)]
                            continue
                        elif l.strip()=="":
                            description=[]
                            continue
                        else:
                            name=pkg+"_"+l.strip()
                            continue
                if body and name:
                    yield {
                        "name"        : name,
                        "body"        : "\n".join(body),
                        "description" : "\n".join(description)
                    }
        loaded_rules={}
        for r in EventCorrelationRule.objects.filter(is_builtin=True):
            loaded_rules[r.name]=None
        # Retrieve rules
        for dirpath,dirnames,filenames in os.walk(os.path.join("fm","rules","correlation")):
            for f in [f for f in filenames if f.endswith(".krb")]:
                for r in get_rules(os.path.join(dirpath,f)):
                    try:
                        rule=EventCorrelationRule.objects.get(name=r["name"])
                        rule.rule=r["body"]
                        rule.description=r["description"]
                        rule.is_builtin=True
                        print "UPDATE CORRELATION RULE %s"%r["name"]
                    except EventCorrelationRule.DoesNotExist:
                        rule=EventCorrelationRule(name=r["name"],rule=r["body"],description=r["description"],is_builtin=True)
                        print "CREATE CORRELATION RULE %s"%r["name"]
                    rule.save()
                    if r["name"] in loaded_rules:
                        del loaded_rules[r["name"]]
        # Delete stale rules
        for r in loaded_rules:
            rule=EventCorrelationRule.objects.get(name=r)
            rule.delete()
            print "DELETE CORRELATION RULE %s"%r
    ##
    ## Syncronize built-in MIBs
    ##
    def sync_mibs(self):
        # Loaded MIBs cache
        loaded_mibs={}
        for m in MIB.objects.all():
            loaded_mibs[m.name]=None
        # Enumerate local stored MIBs
        prefix=os.path.join("share","mibs")
        new_mibs={}
        for m in os.listdir(prefix):
            mib_name,ext=os.path.splitext(m)
            if mib_name not in loaded_mibs:
                # Try to upload new MIB
                try:
                    MIB.load(os.path.join(prefix,m))
                    loaded_mibs[mib_name]=None
                    print "CREATE MIB %s"%mib_name
                except MIBRequiredException,x:
                    new_mibs[mib_name]=x.requires_mib
        # Try to load new MIBs
        while new_mibs:
            l_new_mibs=len(new_mibs)
            for mib_name,requires_mib in new_mibs.items():
                if requires_mib in loaded_mibs:
                    try:
                        MIB.load(os.path.join(prefix,mib_name+".mib"))
                        loaded_mibs[mib_name]=None
                        print "CREATE MIB %s"%mib_name
                        del new_mibs[mib_name]
                    except MIBRequiredException,x:
                        new_mibs[mib_name]=x.requires_mib
            if len(new_mibs)==l_new_mibs: # No new MIBs loaded
                raise Exception("Following builtin MIBs cannot be loaded: %s"%" ".join(new_mibs.keys()))



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Event Collector Interface
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import logging,time

class EventCollector(object):
    name="EventCollector"
    def __init__(self,activator):
        self.activator=activator
    
    def debug(self,msg):
        logging.debug("[%s] %s"%(self.name,msg))
    
    def info(self,msg):
        logging.info("[%s] %s"%(self.name,msg))
    
    def error(self,msg):
        logging.error("[%s] %s"%(self.name,msg))
        
    def check_source_address(self,ip):
        if not self.activator.check_event_source(ip):
            self.error("Invalid event source %s"%ip)
            # Generate "Invalid event source" Event
            body={
                "source"   : "system",
                "component": "noc-activator",
                "activator": self.activator.activator_name,
                "collector": self.name,
                "type"     : "Invalid Event Source",
                "ip"       : ip
            }
            self.process_event(int(time.time()),"",body)
            return False
        return True
        
    def process_event(self,timestamp,ip,body={}):
        self.activator.on_event(timestamp,ip,body)




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report

class Report(noc.main.report.Report):
    name="fm.classification_quality"
    title="Event Classification Quality"
    requires_cursor=True
    columns=[
        Column(""),
        Column("Count",align="RIGHT")]
    
    def get_queryset(self):
        count=self.execute("SELECT COUNT(*) FROM fm_event")[0][0]
        classified=self.execute("""SELECT COUNT(*)
        FROM fm_event
        WHERE event_class_id NOT IN (
            SELECT id FROM fm_eventclass
            WHERE name IN ('DEFAULT','SNMP Trap','SYSLOG')
            )
        """)[0][0]
        if count==0:
            quality=100
        else:
            quality=classified*100/count
        return [
            ["Total events",count],
            ["Classified events",classified],
            ["Classification quality","%8.2f%%"%(quality)]
        ]




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *

class Migration:
    depends_on=(
        ("sa","0008_copy_objects"),
    )
    def forwards(self):
        "Write your forwards migration here"
        db.execute("DROP INDEX cm_config_managed_object_id")
        db.execute("CREATE UNIQUE INDEX cm_config_managed_object_id ON cm_config(managed_object_id)")
        db.delete_column("cm_objectnotify","category_id")
        db.delete_column("cm_objectnotify","location_id")
        for column in ["activator_id","profile_name","scheme","address","port","user","password","super_password","remote_path","trap_source_ip","trap_community"]:
            db.delete_column("cm_config",column)
        for table in ["cm_config","cm_rpsl","cm_dns","cm_prefixlist"]:
            db.delete_column(table,"location_id")
            db.drop_table("%s_categories"%table)
        db.drop_table("cm_object_categories")
        db.drop_table("cm_objectaccess")
        db.execute("DELETE FROM cm_objectcategory")
        db.drop_table("cm_objectcategory")
        db.drop_table("cm_objectlocation")
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from noc.ip.models import VRFGroup,VRF,IPv4BlockAccess,IPv4Block,IPv4Address

class VRFGroupAdmin(admin.ModelAdmin):
    list_display=["name","unique_addresses"]
    search_fields=["name"]
        
class VRFAdmin(admin.ModelAdmin):
    list_display=["rd","name","vrf_group"]
    search_fields=["name","rd"]
    list_filter=["vrf_group"]

class IPv4BlockAccessAdmin(admin.ModelAdmin):
    list_display=["user","vrf","prefix"]
    list_filter=["user","vrf"]
    search_fields=["user","prefix"]

class IPv4BlockAdmin(admin.ModelAdmin):
    list_display=["prefix","vrf","description"]
    list_filter=["vrf","asn"]
    search_fields=["prefix","description"]

class IPv4AddressAdmin(admin.ModelAdmin):
    list_display=["ip","fqdn","vrf","description"]
    list_filter=["vrf"]
    search_fields=["ip","fqdn","description"]

admin.site.register(VRFGroup,VRFGroupAdmin)
admin.site.register(VRF,VRFAdmin)
admin.site.register(IPv4BlockAccess,IPv4BlockAccessAdmin)
admin.site.register(IPv4Block,IPv4BlockAdmin)
admin.site.register(IPv4Address,IPv4AddressAdmin)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django URL dispatcher for module PEER
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.conf.urls.defaults import *
from django.contrib.auth.decorators import login_required
from noc.peer.views import as_rpsl,as_set_rpsl,as_dot,lg,lg_json,inet_rtr_rpsl,person_rpsl,maintainer_rpsl

urlpatterns = patterns ( "",
    (r"^AS/(?P<asn>\d+)/rpsl/",                   login_required(as_rpsl)),
    (r"^AS/(?P<asn>\d+)/dot/",                    login_required(as_dot)),
    (r"^AS-SET/(?P<as_set>AS-[A-Z0-9\-]+)/rpsl/", login_required(as_set_rpsl)),
    (r"^INET-RTR/(?P<router>[A-Za-z0-9\-.]+)/rpsl/", login_required(inet_rtr_rpsl)),
    (r"^person/(?P<person_id>\d+)/rpsl/",         login_required(person_rpsl)),
    (r"^maintainer/(?P<mnt_id>\d+)/rpsl/",        login_required(maintainer_rpsl)),
    (r"lg/(?P<task_id>\d+)/$",                    lg_json),
    (r"lg/",                                      lg),
)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""

from django.contrib import admin
from django import forms
from noc.cm.models import ObjectNotify,Config,DNS,PrefixList,RPSL
from noc.sa.profiles import profile_registry

class ObjectNotifyAdminForm(forms.ModelForm):
    class Meta:
        model=ObjectNotify
    def clean_emails(self):
        emails=self.cleaned_data["emails"]
        return " ".join(emails.replace(";"," ").replace(","," ").split())
        
class ObjectNotifyAdmin(admin.ModelAdmin):
    form=ObjectNotifyAdminForm
    list_display=["type","group","administrative_domain","notify_immediately","notify_delayed","emails"]
    list_filter=["type","group","administrative_domain"]

class ObjectAdmin(admin.ModelAdmin):
    object_class=None
    fields=["pull_every"]

class ConfigAdmin(ObjectAdmin):
    list_display=["repo_path","pull_every","last_modified","next_pull","view_link"]
    search_fields=["repo_path"]
    fields=["pull_every","next_pull"]
    object_class=Config
    def has_change_permission(self,request,obj=None):
        if obj:
            return obj.has_access(request.user)
        else:
            return admin.ModelAdmin.has_delete_permission(self,request)
            
    def has_delete_permission(self,request,obj=None):
        return self.has_change_permission(request,obj)
        
    def queryset(self,request):
        return self.object_class.queryset(request.user)
        
    def save_model(self, request, obj, form, change):
        if obj.has_access(request.user):
            admin.ModelAdmin.save_model(self,request,obj,form,change)
        else:
            raise "Permission denied"
    
    
class DNSAdmin(ObjectAdmin):
    list_display=["repo_path","last_modified","view_link"]
    search_fields=["repo_path"]
    object_class=DNS
    
class PrefixListAdmin(ObjectAdmin):
    list_display=["repo_path","last_modified","view_link"]
    search_fields=["repo_path"]
    object_class=PrefixList
    
class RPSLAdmin(ObjectAdmin):
    list_display=["repo_path","last_modified","view_link"]
    search_fields=["repo_path"]
    object_class=RPSL

admin.site.register(ObjectNotify,   ObjectNotifyAdmin)
admin.site.register(Config,         ConfigAdmin)
admin.site.register(DNS,            DNSAdmin)
admin.site.register(PrefixList,     PrefixListAdmin)
admin.site.register(RPSL,           RPSLAdmin)





from south.db import db
from noc.fm.models import *

class Migration:
    def forwards(self):
        # Mock Models
        Event = db.mock_model(model_name='Event', db_table='fm_event', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'EventLog'
        db.create_table('fm_eventlog', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('event', models.ForeignKey(Event,verbose_name=Event)),
            ('timestamp', models.DateTimeField("Timestamp")),
            ('from_status', models.CharField("From Status",max_length=1,choices=EVENT_STATUS_CHOICES)),
            ('to_status', models.CharField("To Status",max_length=1,choices=EVENT_STATUS_CHOICES)),
            ('message', models.TextField("Message"))
        ))
        db.send_create_signal('fm', ['EventLog'])
    
    def backwards(self):
        db.delete_table('fm_eventlog')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

class Migration:
    
    def forwards(self):
        if db.execute("SELECT COUNT(*) FROM peer_maintainer")[0][0]==0:
            rir_id=db.execute("SELECT id FROM peer_rir LIMIT 1")[0][0]
            db.execute("INSERT INTO peer_maintainer(maintainer,description,auth,rir_id) VALUES(%s,%s,%s,%s)",
                ["Default maintainer","Please change to your maintainer","NO AUTH",rir_id]
            )
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.add_column("fm_eventclassificationrule","drop_event",models.BooleanField("Drop Event",default=False))
    
    def backwards(self):
        db.delete_column("fm_eventclassificationrule","drop_event")



#!/usr/bin/env python
#
# Service Activation Engine
#
import os,sys,getopt

if __name__ == '__main__':
    d=os.path.dirname(sys.argv[0])
    sys.path.insert(0,os.path.join(d,"..",".."))
    sys.path.insert(0,os.path.join(d,".."))
    sys.path.insert(0,d)
    try:
        import settings
    except ImportError:
        sys.stderr.write("Error: Can't find file 'settings.py'. (If the file settings.py does indeed exist, it's causing an ImportError somehow.)\n")
        sys.exit(1)
    os.environ['DJANGO_SETTINGS_MODULE']="noc.settings"
    from noc.sa.sae import SAE
    sae=SAE()
    sae.process_command()


# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report
from noc.lib.version import get_version
import os,sys
import django

class Report(noc.main.report.Report):
    name="main.system"
    title="System version"
    requires_cursor=False
    columns=[Column("Component"),Column("Version")]
    def get_queryset(self):
        return [
            ["NOC"    , get_version()],
            ["OS"     , " ".join(os.uname())],
            ["Python" , sys.version],
            ["Django" , "%s"%str(django.VERSION)],
        ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django's standard views module
## for IP space management module
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.shortcuts import get_object_or_404
from django.http import HttpResponse,HttpResponseRedirect,HttpResponseForbidden
from django import forms

from noc.ip.models import VRFGroup,VRF,IPv4BlockAccess,IPv4Block,IPv4Address
from noc.peer.models import AS
from noc.lib.render import render,render_success
from noc.lib.validators import is_rd,is_cidr,is_int,is_ipv4,is_fqdn
from noc.lib.ip import normalize_prefix,contains
import csv,cStringIO,datetime,subprocess

##
## VRF list
##
def index(request):
    search_by_name=None
    search_by_rd=None
    query=None
    if request.POST:
        if request.POST["search"]:
            query=request.POST["search"]
            if is_rd(query):
                search_by_rd=query
            else:
                search_by_name=query
    l=[]
    for vg in VRFGroup.objects.all():
        if search_by_name:
            vrfs=vg.vrf_set.filter(name__icontains=search_by_name)
        elif search_by_rd:
            vrfs=vg.vrf_set.filter(rd__exact=search_by_rd)
        else:
            vrfs=vg.vrf_set.all()
        if len(vrfs)>0:
            l.append((vg,vrfs.order_by("name")))
    if query is None:
        query=""
    return render(request,"ip/index.html",{"groups":l,"query":query})
##
## Allocated blocks in VRF
##
def vrf_index(request,vrf_id,prefix="0.0.0.0/0"):
    vrf_id=int(vrf_id)
    vrf=get_object_or_404(VRF,id=vrf_id)
    parents=vrf.prefix(prefix).parents
    prefixes=vrf.prefixes(prefix)
    can_allocate=IPv4BlockAccess.check_write_access(request.user,vrf,prefix)
    prefix=vrf.prefix(prefix)
    return render(request,"ip/vrf_index.html",{"vrf":vrf,"parents":parents,"prefixes":prefixes,"prefix":prefix,
                        "can_allocate":can_allocate})
##
## Allocate new block handler
##
class AllocateBlockForm(forms.Form):
    prefix=forms.CharField(label="prefix",required=True)
    description=forms.CharField(label="description",required=True)
    asn=forms.ModelChoiceField(label="ASN",queryset=AS.objects.all(),required=True)
    tt=forms.IntegerField(label="TT #",required=False)
    def clean_prefix(self):
        if not is_cidr(self.cleaned_data["prefix"]):
            raise forms.ValidationError("Invalid prefix")
        return normalize_prefix(self.cleaned_data["prefix"])

def allocate_block(request,vrf_id,prefix=None):
    vrf=get_object_or_404(VRF,id=int(vrf_id))
    if prefix:
        assert is_cidr(prefix)
        block=get_object_or_404(IPv4Block,vrf=vrf,prefix=prefix)
        initial={
            "prefix"      : block.prefix,
            "description" : block.description,
            "asn"         : block.asn.id,
            "tt"          : block.tt
        }
        p="/"+prefix
    else:
        initial={}
        p=""
    if request.POST:
        form=AllocateBlockForm(request.POST)
        if form.is_valid():
            if not IPv4BlockAccess.check_write_access(request.user,vrf,form.cleaned_data["prefix"]):
                return HttpResponseForbidden("Permission denied")
            if prefix:
                block.prefix=form.cleaned_data["prefix"]
                block.description=form.cleaned_data["description"]
                block.asn=form.cleaned_data["asn"]
                block.tt=form.cleaned_data["tt"]
            else:
                block=IPv4Block(vrf=vrf,prefix=form.cleaned_data["prefix"],
                    description=form.cleaned_data["description"],
                    asn=form.cleaned_data["asn"],
                    modified_by=request.user,
                    tt=form.cleaned_data["tt"])
            block.save()
            return HttpResponseRedirect("/ip/%d/%s/"%(vrf.id,block.prefix))
    else:
        form=AllocateBlockForm(initial=initial)
    return render(request,"ip/allocate_block.html",{"vrf":vrf,"form":form,"p":p})
##
## Deallocate block handler
##
def deallocate_block(request,vrf_id,prefix):
    assert is_cidr(prefix)
    vrf_id=int(vrf_id)
    vrf=get_object_or_404(VRF,id=vrf_id)
    block=get_object_or_404(IPv4Block,vrf=vrf,prefix=prefix)
    if not IPv4BlockAccess.check_write_access(request.user,vrf,prefix):
        return HttpResponseForbidden("Permission denied")
    parents=vrf.prefix(prefix).parents
    parent=parents[-1].prefix
    block.delete()
    return HttpResponseRedirect("/ip/%d/%s/"%(vrf_id,parent))
##
## Assign new IP address handler
##
class AssignAddressForm(forms.Form):
    fqdn=forms.CharField(label="FQDN",required=True)
    ip=forms.CharField(label="IP",required=True)
    description=forms.CharField(label="Description",required=False)
    tt=forms.IntegerField(label="TT #",required=False)
    def clean_fqdn(self):
        if not is_fqdn(self.cleaned_data["fqdn"]):
            raise forms.ValidationError("Invalid FQDN")
        return self.cleaned_data["fqdn"].strip()
    def clean_ip(self):
        if not is_ipv4(self.cleaned_data["ip"]):
            raise forms.ValidationError("Invalid IP Address")
        return self.cleaned_data["ip"]
    
def assign_address(request,vrf_id,ip=None):
    vrf=get_object_or_404(VRF,id=int(vrf_id))
    if ip:
        assert is_ipv4(ip)
        address=get_object_or_404(IPv4Address,vrf=vrf,ip=ip)
        initial={
            "fqdn"        : address.fqdn,
            "ip"          : address.ip,
            "description" : address.description,
            "tt"          : address.tt,
        }
        p="/"+ip
    else:
        initial={}
        p=""
    if request.POST:
        form=AssignAddressForm(request.POST)
        if form.is_valid():
            assert is_ipv4(form.cleaned_data["ip"])
            assert is_fqdn(form.cleaned_data["fqdn"])
            if not IPv4BlockAccess.check_write_access(request.user,vrf,form.cleaned_data["ip"]+"/32"):
                return HttpResponseForbidden("Permission denied")
            if ip:
                address.fqdn=form.cleaned_data["fqdn"]
                address.ip=form.cleaned_data["ip"]
                address.description=form.cleaned_data["description"]
                address.tt=form.cleaned_data["tt"]
            else:
                address=IPv4Address(vrf=vrf,fqdn=form.cleaned_data["fqdn"],
                    ip=form.cleaned_data["ip"],description=form.cleaned_data["description"],
                    modified_by=request.user)
            address.save()
            return HttpResponseRedirect("/ip/%d/%s/"%(vrf.id,address.parent.prefix))
    else:
        form=AssignAddressForm(initial=initial)
    return render(request,"ip/assign_address.html",{"vrf":vrf,"form":form,"p":p})
##
## Deallocate ip address handler
##
def revoke_address(request,vrf_id,ip):
    assert is_ipv4(ip)
    vrf_id=int(vrf_id)
    vrf=get_object_or_404(VRF,id=vrf_id)
    address=get_object_or_404(IPv4Address,vrf=vrf,ip=ip)
    if not IPv4BlockAccess.check_write_access(request.user,vrf,ip+"/32"):
        return HttpResponseForbidden("Permission denied")
    prefix=address.parent.prefix
    address.delete()
    return HttpResponseRedirect("/ip/%d/%s/"%(vrf_id,prefix))
##
## An index of tools available for block
##
def block_tools(request,vrf_id,prefix):
    assert is_cidr(prefix)
    vrf_id=int(vrf_id)
    vrf=get_object_or_404(VRF,id=vrf_id)
    if not IPv4BlockAccess.check_write_access(request.user,vrf,prefix):
        return HttpResponseForbidden("Permission denied")
    return render(request,"ip/tools.html",{"vrf":vrf,"prefix":prefix,"upload_ips_form":IPUploadForm(),"upload_ips_axfr_form":AXFRForm()})
##
## Download block's allocated IPs in CSV format
## Columns are: ip,fqdn,description,tt
##
def download_ips(request,vrf_id,prefix):
    assert is_cidr(prefix)
    vrf_id=int(vrf_id)
    vrf=get_object_or_404(VRF,id=vrf_id)
    if not IPv4BlockAccess.check_write_access(request.user,vrf,prefix):
        return HttpResponseForbidden("Permission denied")
    block=get_object_or_404(IPv4Block,vrf=vrf,prefix=prefix)
    out=cStringIO.StringIO()
    writer=csv.writer(out)
    for a in block.addresses:
        writer.writerow([a.ip,a.fqdn,a.description,a.tt])
    return HttpResponse(out.getvalue(),mimetype="text/csv")
##
## Upload allocated IPs in CSV format
## CSV should contain two to four columns rows (ip,fqdn, description, tt)
##
class IPUploadForm(forms.Form):
    file=forms.FileField()
    
def upload_ips(request,vrf_id,prefix):
    def upload_csv(file):
        count=0
        reader=csv.reader(file)
        for row in reader:
            if len(row)<2:
                continue
            if not is_ipv4(row[0]) or not is_fqdn(row[1]):
                continue
            # Leave only addresses residing into "prefix"
            # To prevent uploading to not-owned blocks
            if not contains(prefix,row[0]):
                continue
            changed=False
            try:
                a=IPv4Address.objects.get(vrf=vrf,ip=row[0])
            except IPv4Address.DoesNotExist:
                a=IPv4Address(vrf=vrf,ip=row[0])
                changed=True
            if a.fqdn!=row[1]:
                a.fqdn=row[1]
                changed=True
            if len(row)>=3 and row[2]:
                a.description=row[2]
                changed=True
            if len(row)>=4 and row[3]:
                a.tt=row[3]
                changed=True
            if changed:
                a.modified_by=request.user
                a.last_modified=datetime.datetime.now()
                a.save()
                count+=1
        return count
    assert is_cidr(prefix)
    vrf_id=int(vrf_id)
    vrf=get_object_or_404(VRF,id=vrf_id)
    if not IPv4BlockAccess.check_write_access(request.user,vrf,prefix):
        return HttpResponseForbidden("Permission denied")
    if request.method=="POST":
        form = IPUploadForm(request.POST, request.FILES)
        if form.is_valid():
            count=upload_csv(request.FILES['file'])
            return render_success(request,"CSV file uploaded","%d IP addresses updated"%count)
    return HttpResponseRedirect("/ip/%d/%s/tools/"%(vrf_id,prefix))
##
## Import IP addresses from zone transfer
##
class AXFRForm(forms.Form):
    ns=forms.CharField()
    zone=forms.CharField()
    source_address=forms.IPAddressField(required=False)

def upload_axfr(request,vrf_id,prefix):
    def upload_axfr(data):
        count=0
        for row in data:
            row=row.strip()
            if row=="" or row.startswith(";"):
                continue
            row=row.split()
            if len(row)!=5 or row[2]!="IN" or row[3]!="PTR":
                continue
            if row[3]=="PTR":
                x=row[0].split(".")
                ip="%s.%s.%s.%s"%(x[3],x[2],x[1],x[0])
                fqdn=row[4]
                if fqdn.endswith("."):
                    fqdn=fqdn[:-1]
            # Leave only addresses residing into "prefix"
            # To prevent uploading to not-owned blocks
            if not contains(prefix,ip):
                continue
            changed=False
            try:
                a=IPv4Address.objects.get(vrf=vrf,ip=ip)
            except IPv4Address.DoesNotExist:
                a=IPv4Address(vrf=vrf,ip=ip)
                changed=True
            if a.fqdn!=fqdn:
                a.fqdn=fqdn
                changed=True
            if changed:
                a.modified_by=request.user
                a.last_modified=datetime.datetime.now()
                a.save()
                count+=1
        return count
    assert is_cidr(prefix)
    vrf_id=int(vrf_id)
    vrf=get_object_or_404(VRF,id=vrf_id)
    if not IPv4BlockAccess.check_write_access(request.user,vrf,prefix):
        return HttpResponseForbidden("Permission denied")
    if request.POST:
        form=AXFRForm(request.POST)
        if form.is_valid():
            opts=[]
            if form.cleaned_data["source_address"]:
                opts+=["-b",form.cleaned_data["source_address"]]
            pipe = subprocess.Popen(["dig"]+opts+["axfr","@%s"%form.cleaned_data["ns"],form.cleaned_data["zone"]],
                shell=False, stdout=subprocess.PIPE).stdout
            data=pipe.read()
            pipe.close()
            count=upload_axfr(data.split("\n"))
            return render_success(request,"Zone transfered","%d IP addresses updated"%count)
    return HttpResponseRedirect("/ip/%d/%s/tools/"%(vrf_id,prefix))



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Initial migration for VC application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.vc.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'VCDomain'
        db.create_table('vc_vcdomain', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64,unique=True)),
            ('description', models.TextField("Description",blank=True,null=True))
        ))
        
        # Mock Models
        VCDomain = db.mock_model(model_name='VCDomain', db_table='vc_vcdomain', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'VC'
        db.create_table('vc_vc', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('vc_domain', models.ForeignKey(VCDomain,verbose_name="VC Domain")),
            ('type', models.CharField("Type",max_length=1)),
            ('l1', models.IntegerField("Label 1")),
            ('l2', models.IntegerField("Label 2",default=0)),
            ('description', models.CharField("Description",max_length=256))
        ))
        db.create_index('vc_vc', ['vc_domain_id','type','l1','l2'], unique=True, db_tablespace='')
        
        
        db.send_create_signal('vc', ['VCDomain','VC'])
    
    def backwards(self):
        db.delete_table('vc_vc')
        db.delete_table('vc_vcdomain')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## HttpResponse wrappers and shortcuts
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.http import HttpResponse
from django.utils.simplejson.encoder import JSONEncoder

##
## Render template within context
##
def render(request,template,dict={}):
    return render_to_response(template,dict,context_instance=RequestContext(request))
##
## Create plain text response
##
def render_plain_text(text):
    return HttpResponse(text,mimetype="text/plain")
##
## Create serialized JSON-encoded response
##
def render_json(obj):
    return HttpResponse(JSONEncoder(ensure_ascii=False).encode(obj),mimetype="text/json")
##
## Render "success" page
##
def render_success(request,subject=None,text=None):
    return render(request,"main/success.html",{"subject":subject,"text":text})
##
## Render "failure" page
##
def render_failure(request,subject=None,text=None):
    return render(request,"main/failure.html",{"subject":subject,"text":text})



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco IOS Configuraion-related rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.config import *
from noc.fm.rules.classes.default import DROP

##
## Cisco.IOS Config Changed SNMP
##
class Cisco_IOS_Config_Changed_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS Config Changed SNMP"
    event_class=ConfigChanged
    preference=1000
    required_mibs=["CISCO-CONFIG-MAN-MIB"]
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.43\.2\.0\.1$"), # snmpTrapOID.0 == ciscoConfigManEvent
        (r"^1.3.6.1.4.1.9.9.43.1.1.6.1.5.\d+",r"3"), # ccmHistoryEventConfigDestination == running
        (r"^source$",r"^SNMP Trap$"),
    ]
##
## Cisco.IOS Config Event
## Drop all events except saving to running-config
##
class Cisco_IOS_Config_Event_Drop_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS Config Event Drop SNMP"
    event_class=DROP
    preference=90000
    drop_event=True
    required_mibs=["CISCO-CONFIG-MAN-MIB"]
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.43\.2\.0\.1$"), # snmpTrapOID.0 == ciscoConfigManEvent
        (r"^source$",r"^SNMP Trap$"),
    ]
##
## Cisco.IOS Config Changed Syslog
##
class Cisco_IOS_Config_Changed_Syslog_Rule(ClassificationRule):
    name="Cisco.IOS Config Changed Syslog"
    event_class=ConfigChanged
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%SYS-5-CONFIG_I: Configured"),
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'Task'
        db.create_table('sa_task', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('task_id', models.IntegerField("Task",unique=True)),
            ('start_time', models.DateTimeField("Start Time",auto_now_add=True)),
            ('end_time', models.DateTimeField("End Time")),
            ('profile_name', models.CharField("Profile",max_length=64)),
            ('stream_url', models.CharField("Stream URL",max_length=128)),
            ('action', models.CharField("Action",max_length=64)),
            ('args', models.TextField("Args")),
            ('status', models.CharField("Status",max_length=1,choices=[("n","New"),("p","In Progress"),("f","Failure"),("c","Complete")])),
            ('out', models.TextField("Out"))
        ))
        
        db.send_create_signal('sa', ['Task'])
    
    def backwards(self):
        db.delete_table('sa_task')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco.IOS-specific QoS Events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## Cisco.IOS Auto-QoS Trust Device Detected
##
class CiscoIOSAutoQoSTrustDeviceDetected(EventClass):
    name     = "Cisco.IOS AutoQoS Trust Device Detected"
    category = "NETWORK"
    priority = "NORMAL"
    subject_template="Auto-QoS Trust Device Detected: {{interface}}"
    body_template="""Auto-QoS detected trusted device {{device}} on {{interface}}. Setting port to trusted"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    class Vars:
        interface=Var(required=True)
        device=Var(required=False)
##
## Cisco.IOS Auto-QoS Trust Device Detected
##
class CiscoIOSAutoQoSTrustDeviceLost(EventClass):
    name     = "Cisco.IOS AutoQoS Trust Device Lost"
    category = "NETWORK"
    priority = "NORMAL"
    subject_template="Auto-QoS Trust Device Lost: {{interface}}"
    body_template="""Auto-QoS lost trusted device {{device}} on {{interface}}. Setting port to untrusted"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    class Vars:
        interface=Var(required=True)
        device=Var(required=False)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django URL dispatcher for module FM
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.conf.urls.defaults import *
from django.contrib.auth.decorators import login_required
from noc.fm.views import index,event,reclassify_event,open_event,close_event,create_rule,event_list_css,view_rules,\
    py_event_class,py_event_classification_rule,upload_mib,lookup_managed_object,lookup_events,reload_classifier_config

urlpatterns = patterns ( "",
    (r"^(?P<event_id>\d+)/status/reclassify/$", login_required(reclassify_event)),
    (r"^(?P<event_id>\d+)/status/open/$",       login_required(open_event)),
    (r"^(?P<event_id>\d+)/status/close/$",      login_required(close_event)),
    (r"^(?P<event_id>\d+)/create_rule/$", login_required(create_rule)),
    (r"^(?P<event_id>\d+)/$", login_required(event)),
    (r"^event_list_css/$",    login_required(event_list_css)),
    (r"^view_rules/$",        login_required(view_rules)),
    (r"^py_event_class/(?P<event_class_id>\d+)/$", login_required(py_event_class)),
    (r"^py_event_classification_rule/(?P<rule_id>\d+)/$", login_required(py_event_classification_rule)),
    (r"^upload_mib/$",        login_required(upload_mib)),
    (r"^$",                   login_required(index)),
    (r"^lookup/managed_object/", lookup_managed_object),
    (r"^lookup/events/",      lookup_events),
    (r"^reload_classifier_config", login_required(reload_classifier_config))
)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        if db.execute("SELECT COUNT(*) FROM sa_activator")[0][0]==0:
            db.execute("INSERT INTO sa_activator(name,ip,is_active,auth) VALUES('default','127.0.0.1',true,'xxxxxxxxxxx')")
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from base import *

class IGetMACAddressTable(Interface):
    interface=REStringParameter(r"^\S+$",required=False)
    vlan=VLANIDParameter(required=False)
    mac=MACAddressParameter(required=False)
    returns=ListOfParameter(element=DictParameter(attrs={"vlan_id"    : VLANIDParameter(),
                                                         "mac"        : MACAddressParameter(),
                                                         "interfaces" : StringListParameter(),
                                                         "type"       : StringParameter(), # choices=["D","S"]
                                                         }))
    



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'Activator'
        db.create_table('sa_activator', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('ip', models.IPAddressField("IP")),
            ('auth', models.CharField("Auth String",max_length=64)),
            ('is_active', models.BooleanField("Is Active",default=True))
        ))
        
        db.send_create_signal('sa', ['Activator'])
    
    def backwards(self):
        db.delete_table('sa_activator')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## EIGRP Events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## EIGRP Neighbor Up
##
class EIGRPNeighborUp(EventClass):
    name     = "EIGRP Neighbor Up"
    category = "NETWORK"
    priority = "NORMAL"
    subject_template="EIGRP Neighbor {{ip}} ({{interface}}) is up"
    body_template="""EIGRP Neighbor {{ip}} ({{interface}}) is up"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger=None
    class Vars:
        ip=Var(required=True,repeat=False)
        interface=Var(required=True,repeat=False)
##
## EIGRP Neighbor Down
##
class EIGRPNeighborDown(EventClass):
    name     = "EIGRP Neighbor Down"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="EIGRP Neighbor {{ip}} ({{interface}}) is down"
    body_template="""EIGRP Neighbor {{ip}} ({{interface}}) is down"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger=None
    class Vars:
        ip=Var(required=True,repeat=False)
        interface=Var(required=True,repeat=False)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## System classification rules
## System event classes are generated by NOC itself
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.system import *

##
## Match Unhandled System Exception
##
class UnhandledExceptionRule(ClassificationRule):
    name="Unhandled System Exception"
    event_class=UnhandledException
    patterns=[
        (r"^source$",  r"^system$"),
        (r"^type$",    r"^Unhandled Exception$"),
    ]

##
## Match Invalid Event Source
##
class InvalidEventSourceRule(ClassificationRule):
    name="Invalid Event Source"
    event_class=InvalidEventSource
    patterns=[
        (r"^source$",  r"^system$"),
        (r"^type$",    r"^Invalid Event Source$"),
    ]

##
## Match Periodic Task Failed
##
class PeriodicTaskFailedRule(ClassificationRule):
    name="Periodic Task Failed"
    event_class=PeriodicTaskFailed
    patterns=[
        (r"^source$",  r"^system$"),
        (r"^type$",    r"^Periodic Task Failed$"),
    ]
##
## Match Object unreachable event
##
class ObjectUnreachableRule(ClassificationRule):
    name="Object Unreachable Rule"
    event_class=ObjectUnreachable
    patterns=[
        (r"^source$",  r"^system$"),
        (r"^result$",  r"^failed$"),
        (r"^probe$",   r"^ping$")
    ]
##
## Match Object reachable event
##
class ObjectReachableRule(ClassificationRule):
    name="Object Reachable Rule"
    event_class=ObjectReachable
    patterns=[
        (r"^source$",  r"^system$"),
        (r"^result$",  r"^success$"),
        (r"^probe$",   r"^ping$")
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.db import models
from django.db.models import Q
from django.contrib.auth.models import User
import datetime,random,cPickle,time
from noc.sa.profiles import profile_registry
from noc.sa.periodic import periodic_registry
from noc.sa.script import script_registry
from noc.sa.protocols.sae_pb2 import TELNET,SSH,HTTP
from noc.main.menu import Menu

profile_registry.register_all()
periodic_registry.register_all()
script_registry.register_all()

##
##
##
class AdministrativeDomain(models.Model):
    class Meta:
        verbose_name="Administrative Domain"
        verbose_name_plural="Administrative Domains"
    name=models.CharField("Name",max_length=32,unique=True)
    description=models.TextField("Description",null=True,blank=True)
    def __unicode__(self):
        return self.name
##
##
##
class ObjectGroup(models.Model):
    class Meta:
        verbose_name="Object Group"
        verbose_name_plural="Object Groups"
    name=models.CharField("Name",max_length=32,unique=True)
    description=models.TextField("Description",null=True,blank=True)
    def __unicode__(self):
        return self.name
##
##
##
class Activator(models.Model):
    class Meta:
        verbose_name="Activator"
        verbose_name_plural="Activators"
    name=models.CharField("Name",max_length=32,unique=True)
    ip=models.IPAddressField("IP")
    auth=models.CharField("Auth String",max_length=64)
    is_active=models.BooleanField("Is Active",default=True)
    def __unicode__(self):
        return self.name
##
##
##
class ManagedObject(models.Model):
    class Meta:
        verbose_name="Managed Object"
        verbose_name_plural="Managed Objects"
    name=models.CharField("Name",max_length=64,unique=True)
    is_managed=models.BooleanField("Is Managed?",default=True)
    administrative_domain=models.ForeignKey(AdministrativeDomain,verbose_name="AdministrativeDomain")
    activator=models.ForeignKey(Activator,verbose_name="Activator")
    profile_name=models.CharField("Profile",max_length=128,choices=profile_registry.choices)
    # Access
    scheme=models.IntegerField("Scheme",choices=[(TELNET,"telnet"),(SSH,"ssh"),(HTTP,"http")])
    address=models.CharField("Address",max_length=64)
    port=models.PositiveIntegerField("Port",blank=True,null=True)
    user=models.CharField("User",max_length=32,blank=True,null=True)
    password=models.CharField("Password",max_length=32,blank=True,null=True)
    super_password=models.CharField("Super Password",max_length=32,blank=True,null=True)
    remote_path=models.CharField("Path",max_length=32,blank=True,null=True)
    trap_source_ip=models.IPAddressField("Trap Source IP",null=True)
    trap_community=models.CharField("Trap Community",blank=True,null=True,max_length=64)
    snmp_ro=models.CharField("RO Community",blank=True,null=True,max_length=64)
    snmp_rw=models.CharField("RW Community",blank=True,null=True,max_length=64)
    # CM
    is_configuration_managed=models.BooleanField("Is Configuration Managed?",default=True)
    repo_path=models.CharField("Repo Path",max_length=128,blank=True,null=True)
    #
    groups=models.ManyToManyField(ObjectGroup,verbose_name="Groups",null=True,blank=True)
    #
    def __unicode__(self):
        return self.name
    def config_link(self):
        try:
            return "<A HREF='/cm/view/config/%d/'>Config</A>"%(self.config.id)
        except:
            return ""
    config_link.short_description="Config"
    config_link.allow_tags=True
    
    def scripts_link(self):
        return "<A HREF='/sa/%d/scripts/'>Scripts</A>"%(self.id)
    scripts_link.short_description="Scripts"
    scripts_link.allow_tags=True
    ##
    ## Access control
    ##
    def has_access(self,user):
        if user.is_superuser:
            return True
        return user.useraccess_set.filter(
                    (Q(administrative_domain__isnull=True)|Q(administrative_domain=self.administrative_domain))\
                    &(Q(group__isnull=True)|Q(group__in=self.groups.all))
                    ).count()>0

    def can_change(self,user,administrative_domain,groups):
        if user.is_superuser:
            return True
        if user.useraccess_set.filter(
                (Q(administrative_domain__isnull=True)|Q(administrative_domain=self.administrative_domain))\
                &Q(group__isnull=True)
                ).count()>0:
            return True
        if groups:
            for g in groups:
                if user.useraccess_set.filter(
                        (Q(administrative_domain__isnull=True)|Q(administrative_domain__isnull=self.administrative_domain))\
                        &Q(group=g)
                        ).count()==0:
                    return False
            return True
        return False
        
    @classmethod
    def queryset(cls,user):
        # Idiotic implementation
        ids=[o.id for o in cls.objects.all() if o.has_access(user)]
        return cls.objects.filter(id__in=ids)
    
    def save(self):
        super(ManagedObject,self).save()
        try:
            config=self.config
        except:
            config=None
        if config is None:
            from noc.cm.models import Config
            config=Config(managed_object=self,repo_path=self.repo_path,pull_every=86400)
            config.save()
    
    def delete(self):
        try:
            config=self.config
        except:
            config=None
        if config:
            config.delete()
        super(ManagedObject,self).delete()

##
##
##
class UserAccess(models.Model):
    class Meta:
        verbose_name="User Access"
        verbose_name_plural="User Access"
    user=models.ForeignKey(User,verbose_name="User")
    administrative_domain=models.ForeignKey(AdministrativeDomain,verbose_name="Administrative Domain",blank=True,null=True)
    group=models.ForeignKey(ObjectGroup,verbose_name="Group",blank=True,null=True)
    def __unicode__(self):
        def q(o):
            if o:
                return o.name
            else:
                return "*"
        return "(%s,%s,%s)"%(self.user.username,q(self.administrative_domain),q(self.group))
##
##
##
class TaskSchedule(models.Model):
    periodic_name=models.CharField("Periodic Task",max_length=64,choices=periodic_registry.choices)
    is_enabled=models.BooleanField("Enabled?",default=False)
    run_every=models.PositiveIntegerField("Run Every (secs)",default=86400)
    retries=models.PositiveIntegerField("Retries",default=1)
    retry_delay=models.PositiveIntegerField("Retry Delay (secs)",default=60)
    timeout=models.PositiveIntegerField("Timeout (secs)",default=300)
    next_run=models.DateTimeField("Next Run",auto_now_add=True)
    retries_left=models.PositiveIntegerField("Retries Left",default=1)

    def __unicode__(self):
        return self.periodic_name

    def _periodic_class(self):
        return periodic_registry[self.periodic_name]
    periodic_class=property(_periodic_class)

    @classmethod
    def get_pending_tasks(cls,exclude=None):
        if exclude:
            TaskSchedule.objects.filter(next_run__lte=datetime.datetime.now(),is_enabled=True).exclude(id__in=exclude).order_by("-next_run")
        else:
            return TaskSchedule.objects.filter(next_run__lte=datetime.datetime.now(),is_enabled=True).order_by("-next_run")
##
## Application Menu
##
class AppMenu(Menu):
    app="sa"
    title="Service Activation"
    items=[
        ("Managed Objects", "/admin/sa/managedobject/", "sa.change_managedobject"),
        ("Task Schedules",  "/admin/sa/taskschedule/",  "sa.change_taskschedule"),
        ("Setup", [
            ("Activators",             "/admin/sa/activator/"            , "sa.change_activator"),
            ("Administrative Domains", "/admin/sa/administrativedomain/" , "sa.change_administrativedomain"),
            ("Object Groups",          "/admin/sa/objectgroup/"          , "sa.change_objectgroup"),
            ("User Access",            "/admin/sa/useraccess/"           , "sa.change_useraccess"),
        ])
    ]




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.execute("UPDATE fm_event SET status='U' WHERE subject IS NULL")
        db.execute("UPDATE fm_event SET status='C' WHERE subject IS NOT NULL AND \"timestamp\"<('now'::timestamp-'1day'::interval)")
        db.execute("UPDATE fm_event SET status='A' WHERE subject IS NOT NULL AND \"timestamp\">=('now'::timestamp-'1day'::interval)")
            
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVersion
import re

rx_ver=re.compile(r"Model:\s+(?P<platform>\S+).+JUNOS Base OS Software Suite \[(?P<version>[^\]]+)\]",re.MULTILINE|re.DOTALL)
rx_snmp_ver=re.compile(r"Juniper Networks, Inc.\s+(?P<platform>\S+).+?JUNOS\s+(?P<version>\S+)")

class Script(noc.sa.script.Script):
    name="Juniper.JUNOS.get_version"
    implements=[IGetVersion]
    def execute(self):
        if self.snmp and self.access_profile.snmp_ro:
            try:
                v=self.snmp.get("1.3.6.1.2.1.1.1.0") # sysDescr.0
                match=rx_snmp_ver.search(v)
                return {
                    "vendor"    : "Juniper",
                    "platform"  : match.group("platform"),
                    "version"   : match.group("version"),
                }
            except self.snmp.TimeOutError:
                pass
        v=self.cli("show version")
        match=rx_ver.search(v)
        return {
            "vendor"    : "Juniper",
            "platform"  : match.group("platform"),
            "version"   : match.group("version"),
        }



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Vendor: Juniper
## OS:     JUNOSe
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.profiles
from noc.sa.protocols.sae_pb2 import TELNET,SSH

class Profile(noc.sa.profiles.Profile):
    name="Juniper.JUNOSe"
    supported_schemes=[TELNET,SSH]
    pattern_unpriveleged_prompt=r"^\S+?>"
    command_super="enable"
    pattern_prompt=r"^\S+?#"
    pattern_more=r"^ --More-- "
    command_more=" "
    #pattern_lg_as_path_list=r"(?<=AS path: )(\d+(?: \d+)*)"
    #pattern_lg_best_path=r"^(\s+[+*].+?\s+Router ID: \S+)"
    config_volatile=["^! Configuration script being generated on.*?^"]
    



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.render import render
from noc.lib.registry import Registry
from django.conf import settings

#
admin_media_prefix=settings.ADMIN_MEDIA_PREFIX

##
##
##
class ReportRegistry(Registry):
    name="ReportRegistry"
    subdir="reports"
    classname="Report"
report_registry=ReportRegistry()


##
## Report Columns
## 'format' is a function accepting value and returning valid html code
##
class Column(object):
    def __init__(self,name,align=None,v_align=None,format=None):
        self.name=name
        self.align=align
        self.v_align=v_align
        self.format=format
        
    def render_header(self):
        return "<TH>%s</TH>"%self.name
        
    def render_cell(self,value):
        if self.format:
            value=self.format(value)
        flags=[]
        if self.align:
            flags+=["ALIGN='%s'"%self.align]
        if self.v_align:
            flags+=["VALIGN='%s'"%self.v_align]
        if flags:
            flags=" "+" ".join(flags)
        else:
            flags=""
        return "<TD%s>%s</TD>"%(flags,value)
##
## Boolean field rendered as checkmark
##
class BooleanColumn(Column):
    def render_cell(self,value):
        if value:
            url=admin_media_prefix+"img/admin/icon-yes.gif"
        else:
            url=admin_media_prefix+"img/admin/icon-no.gif"
        return "<TD><IMG SRC='%s' /></TD>"%url

##
##
##
class ReportBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        report_registry.register(m.name,m)
        return m
##
## Abstract Report
##
class Report(object):
    __metaclass__=ReportBase
    name=None
    form_class=None # Or forms.form descendant
    template="main/report.html"
    title="Generic Report"
    requires_cursor=False
    columns=[]
    
    def __init__(self,request,query=None):
        self.request=request
        self.query=query
        if self.form_class:
            if self.query:
                self.form=self.form_class(query)
            else:
                self.form=self.form_class()
        else:
            self.form=None
        if self.requires_cursor:
            from django.db import connection
            self.cursor = connection.cursor()
            
    def is_valid(self):
        return self.form is None or self.form.is_valid()
    
    def render(self):
        out="<TABLE SUMMARY='%s'>"%self.title
        out+="<TR>"+"".join([c.render_header() for c in self.columns])+"</TR>"
        n=0
        for row in self.get_queryset():
            out+="<TR CLASS='row%d'>"%((n%2)+1)+"".join([c.render_cell(v) for c,v in zip(self.columns,row)])+"</TR>"
            n+=1
        out+="</TABLE>"
        return render(self.request,self.template,{"report":self,"query":self.query,"data":out})
            
    def get_queryset(self):
        return []
        
    def execute(self,sql,args=[]):
        self.cursor.execute(sql,args)
        return self.cursor.fetchall()
##
## Matrix report
##
## get_queryset must return a triple of (column-label,row-label,value)
class MatrixReport(Report):
    def render(self):
        def render_column_label(s):
            return "<BR/>".join(s)
        data={}
        cl={}
        rl={}
        for c,r,v in self.get_queryset():
            data[c,r]=v
            cl[c]=None
            rl[r]=None
        cl=sorted(cl.keys())
        rl=sorted(rl.keys())
        out="<TABLE SUMMARY='%s' BORDER='1'>"%self.title
        out+="<TR><TH></TH>%s</TR>"%"".join(["<TH>%s</TH>"%render_column_label(c) for c in cl])
        n=0
        for r in rl:
            out+="<TR CLASS='row%d'><TD><B>%s</B></TD>"%((n%2)+1,r)
            for c in cl:
                try:
                    out+="<TD>%s</TD>"%data[c,r]
                except KeyError:
                    out+="<TD>&nbsp;</TD>"
            out+="</TR>"
            n+=1
        out+="</TABLE>"
        return render(self.request,self.template,{"report":self,"query":self.query,"data":out})



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        db.add_column("sa_managedobject","snmp_ro",models.CharField("RO Community",blank=True,null=True,max_length=64))
        db.add_column("sa_managedobject","snmp_rw",models.CharField("RW Community",blank=True,null=True,max_length=64))
    
    def backwards(self):
        db.delete_column("sa_managedobject","snmp_ro")
        db.delete_column("sa_managedobject","snmp_rw")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.fm.models import *

EVENT_PRIORITY=[
    (0,    "DEFAULT",  "Unclassified event"),
    (0,    "INFO",     "Informational message"),
    (1000, "NORMAL",   "Normal event. No services affected"),
    (2000, "WARNING",  "Some network services can possible be affected"),
    (3000, "MINOR",    "Single service can be affected"),
    (4000, "MAJOR",    "Some part of network services affected"),
    (5000, "CRITICAL", "Serious part of network services affected"),
]

EVENT_CATEGORY=[
    ("DEFAULT",  "Unclassified"),
    ("SYSTEM",   "System Event"),
    ("NETWORK",  "Network Event"),
    ("SECURITY", "Security Event"),
    ("SERVICE",  "Service Event"),
]

class Migration:
    
    def forwards(self):
        for priority,name,description in EVENT_PRIORITY:
            db.execute("INSERT INTO fm_eventpriority(name,priority,description) VALUES(%s,%s,%s)",[name,priority,description])
        for name,description in EVENT_CATEGORY:
            db.execute("INSERT INTO fm_eventcategory(name,description) VALUES(%s,%s)",[name,description])
        default_priority_id=db.execute("SELECT id FROM fm_eventpriority WHERE NAME=%s",["DEFAULT"])[0][0]
        default_category_id=db.execute("SELECT id FROM fm_eventcategory WHERE NAME=%s",["DEFAULT"])[0][0]
        db.execute("""INSERT INTO fm_eventclass(name,category_id,default_priority_id,variables,subject_template,body_template,last_modified)
            VALUES('DEFAULT',%s,%s,'','Unclassified event','Unclassified event','now')""",[default_category_id,default_priority_id])
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'LIR'
        db.create_table('peer_lir', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("LIR name",unique=True,max_length=64))
        ))
        
        # Mock Models
        LIR = db.mock_model(model_name='LIR', db_table='peer_lir', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'AS'
        db.create_table('peer_as', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('lir', models.ForeignKey(LIR,verbose_name=LIR)),
            ('asn', models.IntegerField("ASN",unique=True)),
            ('description', models.CharField("Description",max_length=64)),
            ('rpsl_header', models.TextField("RPSL Header",null=True,blank=True)),
            ('rpsl_footer', models.TextField("RPSL Footer",null=True,blank=True))
        ))
        # Model 'ASSet'
        db.create_table('peer_asset', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('description', models.CharField("Description",max_length=64)),
            ('members', models.TextField("Members",null=True,blank=True)),
            ('rpsl_header', models.TextField("RPSL Header",null=True,blank=True)),
            ('rpsl_footer', models.TextField("RPSL Footer",null=True,blank=True))
        ))
        # Model 'PeeringPointType'
        db.create_table('peer_peeringpointtype', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True))
        ))
        
        # Mock Models
        PeeringPointType = db.mock_model(model_name='PeeringPointType', db_table='peer_peeringpointtype', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'PeeringPoint'
        db.create_table('peer_peeringpoint', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('hostname', models.CharField("FQDN",max_length=64,unique=True)),
            ('router_id', models.IPAddressField("Router-ID",unique=True)),
            ('type', models.ForeignKey(PeeringPointType,verbose_name="Type")),
            ('communities', models.CharField("Import Communities",max_length=128,blank=True,null=True))
        ))
        # Model 'PeerGroup'
        db.create_table('peer_peergroup', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('description', models.CharField("Description",max_length=64)),
            ('communities', models.CharField("Import Communities",max_length=128,blank=True,null=True)),
            ('max_prefixes', models.IntegerField("Max. Prefixes",default=100))
        ))
        
        # Mock Models
        PeerGroup = db.mock_model(model_name='PeerGroup', db_table='peer_peergroup', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        PeeringPoint = db.mock_model(model_name='PeeringPoint', db_table='peer_peeringpoint', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        AS = db.mock_model(model_name='AS', db_table='peer_as', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'Peer'
        db.create_table('peer_peer', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('peer_group', models.ForeignKey(PeerGroup,verbose_name="Peer Group")),
            ('peering_point', models.ForeignKey(PeeringPoint,verbose_name="Peering Point")),
            ('local_asn', models.ForeignKey(AS,verbose_name="Local AS")),
            ('local_ip', models.IPAddressField("Local IP")),
            ('remote_asn', models.IntegerField("Remote AS")),
            ('remote_ip', models.IPAddressField("Remote IP")),
            ('import_filter', models.CharField("Import filter",max_length=64)),
            ('local_pref', models.IntegerField("Local Pref",null=True,blank=True)),
            ('export_filter', models.CharField("Export filter",max_length=64)),
            ('description', models.CharField("Description",max_length=64,null=True,blank=True)),
            ('tt', models.IntegerField("TT",blank=True,null=True)),
            ('communities', models.CharField("Import Communities",max_length=128,blank=True,null=True)),
            ('max_prefixes', models.IntegerField("Max. Prefixes",default=100))
        ))
        
        db.send_create_signal('peer', ['LIR','AS','ASSet','PeeringPointType','PeeringPoint','PeerGroup','Peer'])
    
    def backwards(self):
        db.delete_table('peer_peer')
        db.delete_table('peer_peergroup')
        db.delete_table('peer_peeringpoint')
        db.delete_table('peer_peeringpointtype')
        db.delete_table('peer_asset')
        db.delete_table('peer_as')
        db.delete_table('peer_lir')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *
from noc.lib.url import URL

class Migration:
    
    def forwards(self):
        db.add_column("cm_object","scheme",models.IntegerField("Scheme",blank=True,null=True,choices=[(0,"telnet"),(1,"ssh")]))
        db.add_column("cm_object","address",models.CharField("Address",max_length=64,blank=True,null=True))
        db.add_column("cm_object","port",models.PositiveIntegerField("Port",blank=True,null=True))
        db.add_column("cm_object","user",models.CharField("User",max_length=32,blank=True,null=True))
        db.add_column("cm_object","password",models.CharField("Password",max_length=32,blank=True,null=True))
        db.add_column("cm_object","super_password",models.CharField("Super Password",max_length=32,blank=True,null=True))
        db.add_column("cm_object","remote_path",models.CharField("Path",max_length=32,blank=True,null=True))
        for id,url in db.execute("SELECT id,stream_url FROM cm_object WHERE stream_url!='ssh://u:p@localhost/'"):
            u=URL(url)
            scheme={"telnet":0,"ssh":1}[u.scheme]
            if u.path=="/":
                u.path=None
            db.execute("UPDATE cm_object SET scheme=%s,address=%s,port=%s,\"user\"=%s,password=%s,remote_path=%s WHERE id=%s",
                [scheme,u.host,u.port,u.user,u.password,u.path,id])
        db.delete_column("cm_object","stream_url")
    
    def backwards(self):
        db.delete_column("cm_object","scheme")
        db.delete_column("cm_object","address")
        db.delete_column("cm_object","port")
        db.delete_column("cm_object","user")
        db.delete_column("cm_object","password")
        db.delete_column("cm_object","super_password")
        db.delete_column("cm_object","remote_path")
